@@ -25,22 +25,10 @@
 #include <list>
 #include <vector>
 
+#include "scim-bridge-output.h"
+
 #include "scim-bridge-agent-client-listener.h"
 #include "scim-bridge-agent-imcontext.h"
-#include "scim-bridge-agent-output.h"
-
-#include "event/scim-bridge-agent-beep-event.h"
-#include "event/scim-bridge-agent-commit-event.h"
-#include "event/scim-bridge-agent-delete-surrounding-text-event.h"
-#include "event/scim-bridge-agent-forward-key-event.h"
-#include "event/scim-bridge-agent-set-preedit-attributes-event.h"
-#include "event/scim-bridge-agent-set-preedit-cursor-position-event.h"
-#include "event/scim-bridge-agent-set-preedit-shown-event.h"
-#include "event/scim-bridge-agent-set-preedit-string-event.h"
-#include "event/scim-bridge-agent-get-surrounding-text-event.h"
-#include "event/scim-bridge-agent-surrounding-text-deleted-event.h"
-#include "event/scim-bridge-agent-surrounding-text-gotten-event.h"
-#include "event/scim-bridge-agent-update-preedit-event.h"
 
 using std::list;
 using std::vector;
@@ -70,7 +58,7 @@ static bool enabled_by_default = false;
 
 static unsigned int imengine_id = 0;
 
-static bool on_the_spot_enabled_globally = true;
+static bool on_the_spot_enabled = true;
 
 /* Class definition */
 class ScimBridgeAgentIMContextImpl: public ScimBridgeAgentIMContext
@@ -113,13 +101,8 @@ class ScimBridgeAgentIMContextImpl: public ScimBridgeAgentIMContext
         ScimBridgeAgentIMContextImpl (ScimBridgeAgentClientListener *new_client_listener);
         ~ScimBridgeAgentIMContextImpl ();
 
-        void free ();
-        void free_partially ();
-
-        bool is_on_the_spot_enabled ();
-
-        bool is_preedit_enabled ();
-        void set_preedit_enabled (bool new_state);
+        scim_bridge_preedit_mode_t get_preedit_mode () const;
+        void set_preedit_mode (scim_bridge_preedit_mode_t new_preedit_mode);
 
         bool is_enabled ();
         void set_enabled (bool new_state);
@@ -136,7 +119,7 @@ class ScimBridgeAgentIMContextImpl: public ScimBridgeAgentIMContext
         bool filter_key_event (const KeyEvent &key_event);
         void forward_key_event (const KeyEvent &key_event);
 
-        void cursor_location_changed (int x, int y);
+        void set_cursor_location (int x, int y);
 
         void update_lookup_table_page_size (int page_size);
         void lookup_table_page_up ();
@@ -146,12 +129,11 @@ class ScimBridgeAgentIMContextImpl: public ScimBridgeAgentIMContext
         void process_helper_event (const String &target_uuid, const String &helper_uuid, const Transaction &trans);
         void panel_move_preedit_caret (int caret_pos);
         void panel_process_key_event (const KeyEvent &key_event);
-        void panel_commit (const WideString &wstr);
+        void panel_commit_string (const WideString &wstr);
         void panel_request_help ();
         void panel_change_factory (const String &uuid);
 
         /* Semi private functions */
-
         void alloc_imengine ();
         void free_imengine ();
 
@@ -163,9 +145,9 @@ class ScimBridgeAgentIMContextImpl: public ScimBridgeAgentIMContext
 
         IMEngineInstancePointer imengine;
 
-        bool on_the_spot_enabled;
         bool enabled;
 
+        scim_bridge_preedit_mode_t preedit_mode;
         bool preedit_shown;
         int preedit_cursor_position;
 
@@ -208,13 +190,16 @@ class ScimBridgeAgentIMContextImpl: public ScimBridgeAgentIMContext
 
         bool get_surrounding_text (WideString &text, int &cursor, int maxlen_before, int maxlen_after);
         bool delete_surrounding_text (int offset, int length);
+        bool replace_surrounding_text (const WideString &string, int cursor_position);
 
 };
 
 /* Implementations */
-void ScimBridgeAgentIMContext::static_initialize (ScimBridgeAgentPanelListenerProtected *new_panel_listener, BackEndPointer backend)
+void ScimBridgeAgentIMContext::static_initialize (ScimBridgeAgentPanelListenerProtected *new_panel_listener, const String &new_language, BackEndPointer new_backend)
 {
-    scim_backend = backend;
+    panel_listener = new_panel_listener;
+    scim_language = new_language;
+    scim_backend = new_backend;
 
     fallback_imengine_factory = scim_backend->get_factory (SCIM_COMPOSE_KEY_FACTORY_UUID);
     if (fallback_imengine_factory.null ()) fallback_imengine_factory = new DummyIMEngineFactory ();
@@ -222,18 +207,13 @@ void ScimBridgeAgentIMContext::static_initialize (ScimBridgeAgentPanelListenerPr
     fallback_imengine = fallback_imengine_factory->create_instance (String ("UTF-8"), 0);
     fallback_imengine->signal_connect_commit_string (slot (&ScimBridgeAgentIMContextImpl::slot_fallback_commit));
 
-    panel_listener = new_panel_listener;
+    set_imengine_shared (imengine_shared);
 }
 
 
 void ScimBridgeAgentIMContext::static_finalize ()
 {
-    for (vector<ScimBridgeAgentIMContextImpl*>::iterator i = imcontexts.begin (); i != imcontexts.end (); ++i) {
-        ScimBridgeAgentIMContext *imcontext = *i;
-        if (imcontext != NULL) delete imcontext;
-    }
-    imcontexts.clear ();
-    free_imcontexts.clear ();
+    panel_listener = NULL;
 
     shared_imengine = NULL;
     fallback_imengine = NULL;
@@ -241,6 +221,13 @@ void ScimBridgeAgentIMContext::static_finalize ()
     fallback_imengine_factory = NULL;
 
     scim_backend = NULL;
+
+    for (vector<ScimBridgeAgentIMContextImpl*>::iterator i = imcontexts.begin (); i != imcontexts.end (); ++i) {
+        ScimBridgeAgentIMContext *imcontext = *i;
+        if (imcontext != NULL) delete imcontext;
+    }
+    imcontexts.clear ();
+    free_imcontexts.clear ();
 }
 
 
@@ -250,6 +237,17 @@ ScimBridgeAgentIMContext *ScimBridgeAgentIMContext::alloc (ScimBridgeAgentClient
 }
 
 
+void ScimBridgeAgentIMContext::free_by_client (const ScimBridgeAgentClientListener *client_listener)
+{
+    for (vector<ScimBridgeAgentIMContextImpl*>::iterator i = imcontexts.begin (); i != imcontexts.end (); ++i) {
+        ScimBridgeAgentIMContext *imcontext = *i;
+        if (imcontext != NULL && imcontext->get_client_listener () == client_listener) {
+            delete imcontext;
+        }
+    }
+}
+
+
 ScimBridgeAgentIMContext *ScimBridgeAgentIMContext::find (scim_bridge_imcontext_id_t imcontext_id)
 {
     if (imcontext_id < 0 || imcontext_id >= static_cast<ssize_t> (imcontexts.size ())) {
@@ -260,20 +258,20 @@ ScimBridgeAgentIMContext *ScimBridgeAgentIMContext::find (scim_bridge_imcontext_
 }
 
 
-bool ScimBridgeAgentIMContext::is_on_the_spot_enabled_globally ()
+bool ScimBridgeAgentIMContext::is_on_the_spot_enabled ()
 {
-    return on_the_spot_enabled_globally;
+    return on_the_spot_enabled;
 }
 
 
-void ScimBridgeAgentIMContext::set_on_the_spot_enabled_globally (bool enabled)
+void ScimBridgeAgentIMContext::set_on_the_spot_enabled (bool enabled)
 {
-    on_the_spot_enabled_globally = enabled;
+    on_the_spot_enabled = enabled;
 }
 
 
 ScimBridgeAgentIMContextImpl::ScimBridgeAgentIMContextImpl (ScimBridgeAgentClientListener *new_client_listener):
-client_listener (new_client_listener), imengine (NULL), on_the_spot_enabled (true), enabled (false),
+client_listener (new_client_listener), imengine (NULL), enabled (false), preedit_mode (PREEDIT_ANY),
 preedit_shown (false), preedit_cursor_position (0)
 {
     if (free_imcontexts.empty ()) {
@@ -290,36 +288,27 @@ preedit_shown (false), preedit_cursor_position (0)
 
 ScimBridgeAgentIMContextImpl::~ScimBridgeAgentIMContextImpl ()
 {
-    free_imcontexts.push_back (id);
-    imcontexts[id] = NULL;
-}
+    if (scim_backend != NULL) {
+        panel_listener->deregister_input_context ();
 
+        preedit_string.clear ();
+        preedit_cursor_position = 0;
+        preedit_shown = false;
+        preedit_attributes.clear ();
 
-void ScimBridgeAgentIMContextImpl::free ()
-{
-    if (get_imengine () != NULL) focus_out ();
-    free_partially ();
-}
+        if (imengine_shared && static_cast<ScimBridgeAgentIMContextImpl*> (shared_imengine->get_frontend_data ()) == this) {
+            shared_imengine->reset ();
+            shared_imengine->set_frontend_data (NULL);
+        } else if (imengine != NULL) {
+            imengine->reset ();
+            free_imengine ();
+        }
 
-
-void ScimBridgeAgentIMContextImpl::free_partially ()
-{
-    panel_listener->remove_input_context ();
-
-    preedit_string.clear ();
-    preedit_cursor_position = 0;
-    preedit_shown = false;
-    preedit_attributes.clear ();
-
-    if (imengine_shared && static_cast<ScimBridgeAgentIMContextImpl*> (shared_imengine->get_frontend_data ()) == this) {
-        shared_imengine->reset ();
-        shared_imengine->set_frontend_data (NULL);
-    } else if (imengine != NULL) {
-        imengine->reset ();
-        free_imengine ();
+        client_listener = NULL;
     }
 
-    client_listener = NULL;
+    free_imcontexts.push_back (id);
+    imcontexts[id] = NULL;
 }
 
 
@@ -397,6 +386,8 @@ void ScimBridgeAgentIMContext::set_imengine_shared (bool shared)
 {
     imengine_shared = shared;
 
+    if (scim_backend == NULL) return;
+
     if (imengine_shared) {
         IMEngineFactoryPointer factory = scim_backend->get_default_factory (scim_language, "UTF-8");
         if (factory != NULL) {
@@ -413,8 +404,10 @@ void ScimBridgeAgentIMContext::set_imengine_shared (bool shared)
             ScimBridgeAgentIMContextImpl *imcontext = *i;
 
             if (imcontext != NULL) {
+                panel_listener->prepare (imcontext->get_id ());
                 imcontext->free_imengine ();
                 panel_listener->register_input_context (shared_imengine->get_factory_uuid ());
+                panel_listener->send ();
             }
         }
     } else {
@@ -428,18 +421,6 @@ void ScimBridgeAgentIMContext::set_imengine_shared (bool shared)
 }
 
 
-const String &ScimBridgeAgentIMContext::get_scim_language ()
-{
-    return scim_language;
-}
-
-
-void ScimBridgeAgentIMContext::set_scim_language (const scim::String &language)
-{
-    scim_language = language;
-}
-
-
 bool ScimBridgeAgentIMContextImpl::is_enabled ()
 {
     return enabled;
@@ -450,6 +431,8 @@ void ScimBridgeAgentIMContextImpl::set_enabled (bool new_state)
 {
     enabled = new_state;
 
+    if (imengine_shared) enabled_by_default = enabled;
+
     if (enabled) {
         IMEngineFactoryPointer factory = scim_backend->get_factory (get_imengine ()->get_factory_uuid ());
         if (factory.null ()) factory = fallback_imengine_factory;
@@ -475,15 +458,26 @@ void ScimBridgeAgentIMContextImpl::set_enabled (bool new_state)
 }
 
 
-bool ScimBridgeAgentIMContextImpl::is_preedit_enabled ()
+scim_bridge_preedit_mode_t ScimBridgeAgentIMContextImpl::get_preedit_mode () const
 {
-    return on_the_spot_enabled;
+    return preedit_mode;
 }
 
 
-void ScimBridgeAgentIMContextImpl::set_preedit_enabled (bool new_state)
+void ScimBridgeAgentIMContextImpl::set_preedit_mode (scim_bridge_preedit_mode_t new_preedit_mode)
 {
-    on_the_spot_enabled = new_state;
+    if (preedit_shown) {
+        if ((preedit_mode == PREEDIT_ANY && on_the_spot_enabled) || preedit_mode == PREEDIT_EMBEDDED) {
+            client_listener->set_preedit_shown (id, false);
+            client_listener->update_preedit (id);
+        } else {
+            panel_listener->hide_preedit ();
+        }
+    }
+
+    preedit_mode = new_preedit_mode;
+
+    if (preedit_shown) show_preedit ();
 }
 
 
@@ -523,12 +517,6 @@ IMEngineInstancePointer ScimBridgeAgentIMContextImpl::get_imengine ()
 }
 
 
-bool ScimBridgeAgentIMContextImpl::is_on_the_spot_enabled ()
-{
-    return on_the_spot_enabled && on_the_spot_enabled_globally;
-}
-
-
 bool ScimBridgeAgentIMContextImpl::filter_key_event (const KeyEvent &key_event)
 {
     IMEngineInstancePointer imengine = get_imengine ();
@@ -581,9 +569,7 @@ void ScimBridgeAgentIMContextImpl::open_imengine (IMEngineFactoryPointer factory
         if (imengine_shared) set_enabled_by_default (true);
         if (get_imengine ()->get_factory_uuid () == factory->get_uuid ()) return;
 
-        IMEngineInstancePointer new_imengine = factory->create_instance ("UTF-8", imengine_id);
-        ++imengine_id;
-
+        IMEngineInstancePointer new_imengine = factory->create_instance ("UTF-8", get_imengine ()->get_id ());
         if (imengine_shared) {
             shared_imengine = new_imengine;
             shared_imengine->set_frontend_data (static_cast<ScimBridgeAgentIMContext*> (this));
@@ -604,7 +590,7 @@ void ScimBridgeAgentIMContextImpl::open_imengine (IMEngineFactoryPointer factory
 
 void ScimBridgeAgentIMContextImpl::beep ()
 {
-    client_listener->push_event (new ScimBridgeAgentBeepEvent (id));
+    client_listener->beep (id);
 }
 
 
@@ -675,23 +661,25 @@ void ScimBridgeAgentIMContextImpl::focus_out ()
 }
 
 
-void ScimBridgeAgentIMContextImpl::cursor_location_changed (int x, int y)
+void ScimBridgeAgentIMContextImpl::set_cursor_location (int x, int y)
 {
-    cursor_x = x;
-    cursor_y = y;
-    panel_listener->update_cursor_location (cursor_x, cursor_y);
+    if (preedit_mode != PREEDIT_FLOATING) {
+        cursor_x = x;
+        cursor_y = y;
+        panel_listener->update_cursor_location (cursor_x, cursor_y);
+    }
 }
 
 
 void ScimBridgeAgentIMContextImpl::show_preedit ()
 {
-    if (is_on_the_spot_enabled ()) {
+    if ((preedit_mode == PREEDIT_ANY && on_the_spot_enabled) || preedit_mode == PREEDIT_EMBEDDED) {
         preedit_shown = true;
-        client_listener->push_event (new ScimBridgeAgentSetPreeditShownEvent (id, true));
-        client_listener->push_event (new ScimBridgeAgentSetPreeditCursorPositionEvent (id, preedit_cursor_position));
-        client_listener->push_event (new ScimBridgeAgentSetPreeditStringEvent (id, preedit_string));
-        client_listener->push_event (new ScimBridgeAgentSetPreeditAttributesEvent (id, preedit_attributes));
-        client_listener->push_event (new ScimBridgeAgentUpdatePreeditEvent (id));
+        client_listener->set_preedit_shown (id, true);
+        client_listener->set_preedit_cursor_position (id, preedit_cursor_position);
+        client_listener->set_preedit_string (id, preedit_string);
+        client_listener->set_preedit_attributes (id, preedit_attributes);
+        client_listener->update_preedit (id);
     } else {
         panel_listener->show_preedit ();
     }
@@ -700,10 +688,10 @@ void ScimBridgeAgentIMContextImpl::show_preedit ()
 
 void ScimBridgeAgentIMContextImpl::hide_preedit ()
 {
-    if (is_on_the_spot_enabled ()) {
+    if ((preedit_mode == PREEDIT_ANY && on_the_spot_enabled) || preedit_mode == PREEDIT_EMBEDDED) {
         if (preedit_shown) {
-            client_listener->push_event (new ScimBridgeAgentSetPreeditShownEvent (id, false));
-            client_listener->push_event (new ScimBridgeAgentUpdatePreeditEvent (id));
+            client_listener->set_preedit_shown (id, false);
+            client_listener->update_preedit (id);
         }
         preedit_shown = false;
         preedit_cursor_position = 0;
@@ -759,11 +747,11 @@ void ScimBridgeAgentIMContextImpl::update_preedit_caret (int cursor_pos)
 
     if (is_on_the_spot_enabled ()) {
         if (!preedit_shown) {
-            client_listener->push_event (new ScimBridgeAgentSetPreeditShownEvent (id, true));
+            client_listener->set_preedit_shown (id, true);
             preedit_shown = true;
         }
-        client_listener->push_event (new ScimBridgeAgentSetPreeditCursorPositionEvent (id, preedit_cursor_position));
-        client_listener->push_event (new ScimBridgeAgentUpdatePreeditEvent (id));
+        client_listener->set_preedit_cursor_position (id, preedit_cursor_position);
+        client_listener->update_preedit (id);
     } else {
         panel_listener->set_preedit_cursor_position (cursor_pos);
     }
@@ -779,14 +767,17 @@ void ScimBridgeAgentIMContextImpl::update_preedit_string (const WideString &str,
 
     if (is_on_the_spot_enabled ()) {
         if (!preedit_shown) {
-            client_listener->push_event (new ScimBridgeAgentSetPreeditShownEvent (id, true));
+            client_listener->set_preedit_shown (id, true);
             preedit_shown = true;
         }
+        client_listener->set_preedit_string (id, preedit_string);
+
+        // This will cause very annoying cursor movements for Japanese.
         //preedit_cursor_position = str.length ();
-        client_listener->push_event (new ScimBridgeAgentSetPreeditStringEvent (id, preedit_string));
-        client_listener->push_event (new ScimBridgeAgentSetPreeditAttributesEvent (id, preedit_attributes));
         //client_listener->push_event (new ScimBridgeAgentSetPreeditCursorPositionEvent (id, preedit_cursor_position));
-        client_listener->push_event (new ScimBridgeAgentUpdatePreeditEvent (id));
+
+        client_listener->set_preedit_attributes (id, preedit_attributes);
+        client_listener->update_preedit (id);
     } else {
         panel_listener->set_preedit_string (str, attrs);
     }
@@ -795,13 +786,13 @@ void ScimBridgeAgentIMContextImpl::update_preedit_string (const WideString &str,
 
 void ScimBridgeAgentIMContextImpl::commit (const WideString &str)
 {
-    client_listener->push_event (new ScimBridgeAgentCommitEvent (id, str));
+    client_listener->commit_string (id, str);
 }
 
 
 void ScimBridgeAgentIMContextImpl::forward_key_event (const KeyEvent &key_event)
 {
-    client_listener->push_event (new ScimBridgeAgentForwardKeyEvent (id, key_event));
+    client_listener->forward_key_event (id, key_event);
 }
 
 
@@ -837,90 +828,30 @@ void ScimBridgeAgentIMContextImpl::send_helper_event (const String &helper_uuid,
 
 bool ScimBridgeAgentIMContextImpl::get_surrounding_text (WideString &string, int &cursor_pos, int max_before, int max_after)
 {
-    static scim_bridge_message_response_id_t response_id = 0;
-    ++response_id;
-
-    timeval previous_time;
-    gettimeofday (&previous_time, NULL);
-
-    client_listener->push_event (new ScimBridgeAgentGetSurroundingTextEvent (response_id, id, max_before, max_after));
-    const ScimBridgeAgentSurroundingTextGottenEvent *response = NULL;
-
-    timeval current_time;
-    do {
-        const ScimBridgeAgentEvent *tmp_event = client_listener->poll_response ();
-        if (tmp_event != NULL && tmp_event->get_code () == SURROUNDING_TEXT_GOTTEN_EVENT) {
-            response = static_cast<const ScimBridgeAgentSurroundingTextGottenEvent*> (tmp_event);
-            if (response->get_response_id () == response_id) {
-                break;
-            } else {
-                response = NULL;
-            }
-        }
-        delete tmp_event;
-
-        // handle timeout;
-        gettimeofday (&current_time, NULL);
-        if ((current_time.tv_sec - previous_time.tv_sec) * 1000000 + (current_time.tv_usec - previous_time.tv_usec) > 1000000) {
-            break;
-        } else {
-            usleep (10);
-        }
-    } while (response == NULL);
-
-    if (response == NULL) {
+    if (client_listener->get_surrounding_string (id, max_before, max_after, string, cursor_pos)) {
         return false;
     } else {
-        string = response->get_surrounding_text ();
-        cursor_pos = response->get_cursor_position ();
-        bool retval = response->get_retval ();
-        delete response;
-        return retval;
+        return true;
     }
 }
 
 
 bool ScimBridgeAgentIMContextImpl::delete_surrounding_text (int offset, int length)
 {
-    static scim_bridge_message_response_id_t response_id = 0;
-    ++response_id;
-
-    timeval previous_time;
-    gettimeofday (&previous_time, NULL);
-
-    client_listener->push_event (new ScimBridgeAgentDeleteSurroundingTextEvent (response_id, id, offset, length));
-    const ScimBridgeAgentSurroundingTextDeletedEvent *response = NULL;
-
-    timeval current_time;
-    do {
-        const ScimBridgeAgentEvent *tmp_event = client_listener->poll_response ();
-        if (tmp_event != NULL && tmp_event->get_code () == SURROUNDING_TEXT_DELETED_EVENT) {
-            response = static_cast<const ScimBridgeAgentSurroundingTextDeletedEvent*> (tmp_event);
-
-            if (response->get_response_id () == response_id) {
-                break;
-            } else {
-                response = NULL;
-            }
-        }
-        delete tmp_event;
-
-        // handle timeout;
-        gettimeofday (&current_time, NULL);
-        if ((current_time.tv_sec - previous_time.tv_sec) * 1000000 + (current_time.tv_usec - previous_time.tv_usec) > 1000000) {
-            break;
-        } else {
-            usleep (10);
-        }
-    } while (response == NULL);
-
-    if (response == NULL) {
+    if (client_listener->delete_surrounding_string (id, offset, length)) {
         return false;
     } else {
-        bool retval = response->get_retval ();
-        if (response->is_reset_needed ()) reset ();
-        delete response;
-        return retval;
+        return true;
+    }
+}
+
+
+bool ScimBridgeAgentIMContextImpl::replace_surrounding_text (const WideString &string, int cursor_position)
+{
+    if (client_listener->replace_surrounding_string (id, string, cursor_position)) {
+        return false;
+    } else {
+        return true;
     }
 }
 
@@ -968,7 +899,7 @@ void ScimBridgeAgentIMContextImpl::panel_move_preedit_caret (int caret_pos)
 }
 
 
-void ScimBridgeAgentIMContextImpl::panel_commit (const WideString &wstr)
+void ScimBridgeAgentIMContextImpl::panel_commit_string (const WideString &wstr)
 {
     commit (wstr);
 }
@@ -978,25 +909,22 @@ void ScimBridgeAgentIMContextImpl::panel_request_help ()
 {
     String help =  String (_ ("Smart Common Input Method platform ")) +
         String (SCIM_VERSION) +
-        String (_ ("\n (C) 2002-2005 James Su <suzhe@tsinghua.org.cn>\n\n"));
+        String (_ ("\n(C) 2002-2005 James Su <suzhe@tsinghua.org.cn>\n\n"));
 
-    help += String (_ ("- Powered by ScimBridge -"));
-    help += String (_ ("\n\n"));
+        IMEngineFactoryPointer factory = scim_backend->get_factory (get_imengine ()->get_factory_uuid ());
+        if (factory.null ()) factory = fallback_imengine_factory;
+            help += utf8_wcstombs (factory->get_name ());
+            help += String (_ (":\n\n"));
 
-    IMEngineFactoryPointer factory = scim_backend->get_factory (get_imengine ()->get_factory_uuid ());
-    if (factory.null ()) factory = fallback_imengine_factory;
-    help += utf8_wcstombs (factory->get_name ());
-    help += String (_ (":\n\n"));
+            help += utf8_wcstombs (factory->get_authors ());
+            help += String (_ ("\n\n"));
 
-    help += utf8_wcstombs (factory->get_authors ());
-    help += String (_ ("\n\n"));
+            help += utf8_wcstombs (factory->get_help ());
+            help += String (_ ("\n\n"));
 
-    help += utf8_wcstombs (factory->get_help ());
-    help += String (_ ("\n\n"));
+            help += utf8_wcstombs (factory->get_credits ());
 
-    help += utf8_wcstombs (factory->get_credits ());
-
-    panel_listener->show_help (help);
+            panel_listener->show_help (help);
 }
 
 
@@ -1009,11 +937,11 @@ void ScimBridgeAgentIMContextImpl::panel_change_factory (const String &uuid)
 /* Slot implementations */
 void ScimBridgeAgentIMContextImpl::slot_show_preedit (IMEngineInstanceBase *imengine)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_show_preedit...");
+    scim_bridge_pdebugln (3, "slot_show_preedit...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->show_preedit ();
     }
@@ -1022,11 +950,11 @@ void ScimBridgeAgentIMContextImpl::slot_show_preedit (IMEngineInstanceBase *imen
 
 void ScimBridgeAgentIMContextImpl::slot_hide_preedit (IMEngineInstanceBase *imengine)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_hide_preedit...");
+    scim_bridge_pdebugln (3, "slot_hide_preedit...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->hide_preedit ();
     }
@@ -1035,11 +963,11 @@ void ScimBridgeAgentIMContextImpl::slot_hide_preedit (IMEngineInstanceBase *imen
 
 void ScimBridgeAgentIMContextImpl::slot_show_aux_string (IMEngineInstanceBase *imengine)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_show_aux_string...");
+    scim_bridge_pdebugln (3, "slot_show_aux_string...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->show_aux_string ();
     }
@@ -1048,11 +976,11 @@ void ScimBridgeAgentIMContextImpl::slot_show_aux_string (IMEngineInstanceBase *i
 
 void ScimBridgeAgentIMContextImpl::slot_hide_aux_string (IMEngineInstanceBase *imengine)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_hide_aux_string...");
+    scim_bridge_pdebugln (3, "slot_hide_aux_string...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->hide_aux_string ();
     }
@@ -1061,11 +989,11 @@ void ScimBridgeAgentIMContextImpl::slot_hide_aux_string (IMEngineInstanceBase *i
 
 void ScimBridgeAgentIMContextImpl::slot_update_aux_string (IMEngineInstanceBase *imengine, const WideString &str, const AttributeList &attrs)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_update_aux_string...");
+    scim_bridge_pdebugln (3, "slot_update_aux_string...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->update_aux_string (str, attrs);
     }
@@ -1074,11 +1002,11 @@ void ScimBridgeAgentIMContextImpl::slot_update_aux_string (IMEngineInstanceBase
 
 void ScimBridgeAgentIMContextImpl::slot_show_lookup_table (IMEngineInstanceBase *imengine)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_show_lookup_table...");
+    scim_bridge_pdebugln (3, "slot_show_lookup_table...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->show_lookup_table ();
     }
@@ -1087,11 +1015,11 @@ void ScimBridgeAgentIMContextImpl::slot_show_lookup_table (IMEngineInstanceBase
 
 void ScimBridgeAgentIMContextImpl::slot_hide_lookup_table (IMEngineInstanceBase *imengine)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_hide_lookup_table...");
+    scim_bridge_pdebugln (3, "slot_hide_lookup_table...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->hide_lookup_table ();
     }
@@ -1100,11 +1028,11 @@ void ScimBridgeAgentIMContextImpl::slot_hide_lookup_table (IMEngineInstanceBase
 
 void ScimBridgeAgentIMContextImpl::slot_update_lookup_table (IMEngineInstanceBase *imengine, const LookupTable &table)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_update_lookup_table...");
+    scim_bridge_pdebugln (3, "slot_update_lookup_table...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->update_lookup_table (table);
     }
@@ -1113,11 +1041,11 @@ void ScimBridgeAgentIMContextImpl::slot_update_lookup_table (IMEngineInstanceBas
 
 void ScimBridgeAgentIMContextImpl::slot_update_preedit_caret (IMEngineInstanceBase *imengine, int cursor_pos)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_update_preedit_caret...");
+    scim_bridge_pdebugln (3, "slot_update_preedit_caret...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->update_preedit_caret (cursor_pos);
     }
@@ -1126,11 +1054,11 @@ void ScimBridgeAgentIMContextImpl::slot_update_preedit_caret (IMEngineInstanceBa
 
 void ScimBridgeAgentIMContextImpl::slot_update_preedit_string (IMEngineInstanceBase *imengine, const WideString &str, const AttributeList &attrs)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_update_preedit_string...");
+    scim_bridge_pdebugln (3, "slot_update_preedit_string...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->update_preedit_string (str, attrs);
     }
@@ -1139,11 +1067,11 @@ void ScimBridgeAgentIMContextImpl::slot_update_preedit_string (IMEngineInstanceB
 
 void ScimBridgeAgentIMContextImpl::slot_commit (IMEngineInstanceBase *imengine, const WideString &str)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_commit_string...");
+    scim_bridge_pdebugln (3, "slot_commit_string...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->commit (str);
     }
@@ -1152,11 +1080,11 @@ void ScimBridgeAgentIMContextImpl::slot_commit (IMEngineInstanceBase *imengine,
 
 void ScimBridgeAgentIMContextImpl::slot_forward_key_event (IMEngineInstanceBase *imengine, const KeyEvent &key_event)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_forward_key_event...");
+    scim_bridge_pdebugln (3, "slot_forward_key_event...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->forward_key_event (key_event);
     }
@@ -1165,11 +1093,11 @@ void ScimBridgeAgentIMContextImpl::slot_forward_key_event (IMEngineInstanceBase
 
 void ScimBridgeAgentIMContextImpl::slot_register_properties (IMEngineInstanceBase *imengine, const PropertyList &properties)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_register_properties...");
+    scim_bridge_pdebugln (3, "slot_register_properties...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->register_properties (properties);
     }
@@ -1178,11 +1106,11 @@ void ScimBridgeAgentIMContextImpl::slot_register_properties (IMEngineInstanceBas
 
 void ScimBridgeAgentIMContextImpl::slot_update_property (IMEngineInstanceBase *imengine, const Property &property)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_update_property...");
+    scim_bridge_pdebugln (3, "slot_update_property...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->update_property (property);
     }
@@ -1191,11 +1119,11 @@ void ScimBridgeAgentIMContextImpl::slot_update_property (IMEngineInstanceBase *i
 
 void ScimBridgeAgentIMContextImpl::slot_beep (IMEngineInstanceBase *imengine)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_beep...");
+    scim_bridge_pdebugln (3, "slot_beep...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->beep ();
     }
@@ -1204,11 +1132,11 @@ void ScimBridgeAgentIMContextImpl::slot_beep (IMEngineInstanceBase *imengine)
 
 void ScimBridgeAgentIMContextImpl::slot_start_helper (IMEngineInstanceBase *imengine, const String &helper_uuid)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_start_helper...");
+    scim_bridge_pdebugln (3, "slot_start_helper...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->start_helper (helper_uuid);
     }
@@ -1217,11 +1145,11 @@ void ScimBridgeAgentIMContextImpl::slot_start_helper (IMEngineInstanceBase *imen
 
 void ScimBridgeAgentIMContextImpl::slot_stop_helper (IMEngineInstanceBase *imengine, const String &helper_uuid)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_stop_helper...");
+    scim_bridge_pdebugln (3, "slot_stop_helper...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->stop_helper (helper_uuid);
     }
@@ -1230,11 +1158,11 @@ void ScimBridgeAgentIMContextImpl::slot_stop_helper (IMEngineInstanceBase *imeng
 
 void ScimBridgeAgentIMContextImpl::slot_send_helper_event (IMEngineInstanceBase *imengine, const String &helper_uuid, const Transaction &trans)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_send_helper_event...");
+    scim_bridge_pdebugln (3, "slot_send_helper_event...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         imcontext->send_helper_event (helper_uuid, trans);
     }
@@ -1243,12 +1171,12 @@ void ScimBridgeAgentIMContextImpl::slot_send_helper_event (IMEngineInstanceBase
 
 bool ScimBridgeAgentIMContextImpl::slot_get_surrounding_text (IMEngineInstanceBase *imengine, WideString &string, int &cursor, int maxlen_before, int maxlen_after)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_get_surrounding_text...");
+    scim_bridge_pdebugln (3, "slot_get_surrounding_text...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
-        return false;
+        scim_bridge_pdebugln (8, "No imcontext is connected");
+            return false;
     } else {
         return imcontext->get_surrounding_text (string, cursor, maxlen_before, maxlen_after);
     }
@@ -1257,12 +1185,12 @@ bool ScimBridgeAgentIMContextImpl::slot_get_surrounding_text (IMEngineInstanceBa
 
 bool ScimBridgeAgentIMContextImpl::slot_delete_surrounding_text (IMEngineInstanceBase *imengine, int offset, int length)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_delete_surrounding_text...");
+    scim_bridge_pdebugln (3, "slot_delete_surrounding_text...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
-        return false;
+        scim_bridge_pdebugln (8, "No imcontext is connected");
+            return false;
     } else {
         return imcontext->delete_surrounding_text (offset, length);
     }
@@ -1271,11 +1199,11 @@ bool ScimBridgeAgentIMContextImpl::slot_delete_surrounding_text (IMEngineInstanc
 
 void ScimBridgeAgentIMContextImpl::slot_fallback_commit (IMEngineInstanceBase *imengine, const WideString &commit_string)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "slot_fallback_commit_string...");
+    scim_bridge_pdebugln (3, "slot_fallback_commit_string...");
 
     ScimBridgeAgentIMContextImpl *imcontext = static_cast<ScimBridgeAgentIMContextImpl*> (imengine->get_frontend_data ());
     if (imcontext == NULL) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 8, "No imcontext is connected");
+        scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
         return imcontext->commit (commit_string);
     }
