@@ -19,8 +19,6 @@
  */
 
 #include <assert.h>
-#include <pthread.h>
-#include <semaphore.h>
 #include <unistd.h>
 
 #include <sys/select.h>
@@ -28,44 +26,27 @@
 #include <sys/time.h>
 #include <sys/types.h>
 
-#include <iostream>
-
-#define Uses_SCIM_BACKEND
 #define Uses_SCIM_IMENGINE
 #define Uses_SCIM_IMENGINE_MODULE
 #define Uses_SCIM_PANEL_CLIENT
 
 #include <scim.h>
 
-#include "scim-bridge-agent-event-server.h"
-#include "scim-bridge-agent-output.h"
+#include "scim-bridge-agent-imcontext.h"
 #include "scim-bridge-agent-panel-listener.h"
 #include "scim-bridge-agent-panel-listener-protected.h"
 #include "scim-bridge-agent-protected.h"
-#include "scim-bridge-display.h"
-#include "scim-bridge-path.h"
 
-#include "event/scim-bridge-agent-panel-pending-event.h"
+#include "scim-bridge-display.h"
+#include "scim-bridge-output.h"
+#include "scim-bridge-path.h"
 
 #include "utils/scim-bridge-agent-panel-client.h"
 
-using std::endl;
-using std::ifstream;
-using std::ofstream;
 using std::vector;
 
 using namespace scim;
 
-/* Type def */
-typedef enum _thread_status_t
-{
-    THREAD_UNINITIALIZED,
-    THREAD_INITIALIZING,
-    THREAD_INITIALIZED,
-    THREAD_FINALIZING,
-    THREAD_FINALIZED
-} thread_status_t;
-
 /* Class definition */
 class ScimBridgeAgentPanelListenerImpl: public ScimBridgeAgentPanelListener
 {
@@ -75,20 +56,17 @@ class ScimBridgeAgentPanelListenerImpl: public ScimBridgeAgentPanelListener
         ScimBridgeAgentPanelListenerImpl (const String &config_name, const ScimBridgeDisplay *new_display, ScimBridgeAgentProtected *new_agent);
         ~ScimBridgeAgentPanelListenerImpl ();
 
-        void read_and_dispatch ();
-        void prepare (scim_bridge_imcontext_id_t imcontext_id);
-        void send ();
+        retval_t initialize ();
 
-        retval_t launch ();
+        int get_socket_fd () const;
 
-        retval_t check_lockfile ();
-        retval_t create_lockfile (pid_t pid);
+        scim_bridge_agent_event_type_t get_trigger_events () const;
+
+        bool handle_event (scim_bridge_agent_event_type_t event_type);
 
         /* The following functions are virtually protected */
-        void initialize ();
-
-        retval_t destroy_lockfile ();
-        retval_t update_lockfile (pid_t pid);
+        void prepare (scim_bridge_imcontext_id_t imcontext_id);
+        void send ();
 
         void focus_in (const String &factory_uuid);
         void focus_out ();
@@ -124,23 +102,7 @@ class ScimBridgeAgentPanelListenerImpl: public ScimBridgeAgentPanelListener
         void show_help (const String &string);
 
         void register_input_context (const String &factory_uuid);
-        void remove_input_context ();
-
-        /* The following functions are virtually private */
-        int get_panel_fd ();
-
-        void connect_panel_client ();
-
-        void panel_error_occured ();
-        void panel_event_occured ();
-        void panel_listener_launched ();
-        void panel_listener_exited ();
-        bool panel_listener_is_exit_triggered ();
-        void read_event ();
-
-    protected:
-
-        void do_close_event_client ();
+        void deregister_input_context ();
 
     private:
 
@@ -149,15 +111,11 @@ class ScimBridgeAgentPanelListenerImpl: public ScimBridgeAgentPanelListener
         ScimBridgeAgentPanelClient *scim_panel_client;
 
         int panel_fd;
-        thread_status_t thread_status;
 
         scim_bridge_imcontext_id_t prepared_imcontext_id;
 
-        pthread_t panel_listener_thread;
-
-        sem_t semaphore;
-
-        bool launched;
+        retval_t open_panel_client ();
+        retval_t close_panel_client ();
 
         /* Panel related callbacks */
         void slot_reload_config (scim_bridge_imcontext_id_t id);
@@ -178,95 +136,23 @@ class ScimBridgeAgentPanelListenerImpl: public ScimBridgeAgentPanelListener
 
 };
 
-/* Helper functions */
-void *run_panel_listener (void *arg)
-{
-    ScimBridgeAgentPanelListenerImpl *panel_listener = static_cast<ScimBridgeAgentPanelListenerImpl*> (arg);
-
-    panel_listener->panel_listener_launched ();
-
-    timeval previous_time;
-    gettimeofday (&previous_time, NULL);
-
-    while (!panel_listener->panel_listener_is_exit_triggered ()) {
-        const int panel_fd = panel_listener->get_panel_fd ();
-
-        if (panel_fd < 0) {
-            panel_listener->connect_panel_client ();
-            sleep (1);
-            continue;
-        }
-
-        timeval timeout;
-        timeout.tv_sec = 60 * 60;
-        timeout.tv_usec = 0;
-
-        fd_set read_set;
-        FD_ZERO (&read_set);
-        FD_SET (panel_fd, &read_set);
-
-        fd_set error_set;
-        FD_ZERO (&error_set);
-        FD_SET (panel_fd, &error_set);
-        const int ret = select (panel_fd + 1, &read_set, NULL, &error_set, &timeout);
-
-        timeval current_time;
-        gettimeofday (&current_time, NULL);
-
-        if ((current_time.tv_sec - previous_time.tv_sec) + (current_time.tv_usec - previous_time.tv_usec) / 1000000 >= 60 * 60) {
-            panel_listener->update_lockfile (-1);
-            previous_time = current_time;
-        }
-
-        if (ret > 0 && !panel_listener->panel_listener_is_exit_triggered ()) {
-            if (FD_ISSET (panel_fd, &error_set)) {
-                panel_listener->panel_error_occured ();
-            } else {
-                panel_listener->panel_event_occured ();
-            }
-        }
-    }
-
-    panel_listener->panel_listener_exited ();
-
-    return NULL;
-}
-
-
 /* Implementations */
 ScimBridgeAgentPanelListener *ScimBridgeAgentPanelListener::alloc (const String &config_name, const ScimBridgeDisplay *new_display, ScimBridgeAgentProtected *new_agent)
 {
     ScimBridgeAgentPanelListenerImpl *panel_listener = new ScimBridgeAgentPanelListenerImpl (config_name, new_display, new_agent);
-    if (panel_listener->check_lockfile ()) {
+    if (panel_listener->initialize ()) {
         delete panel_listener;
         return NULL;
     } else {
-        panel_listener->initialize ();
         return panel_listener;
     }
 }
 
 
-ScimBridgeAgentPanelListenerImpl::ScimBridgeAgentPanelListenerImpl (const String &config_name, const ScimBridgeDisplay *display, ScimBridgeAgentProtected *new_agent): agent (new_agent), panel_fd (-1), thread_status (THREAD_UNINITIALIZED), prepared_imcontext_id (-1), launched (false)
+ScimBridgeAgentPanelListenerImpl::ScimBridgeAgentPanelListenerImpl (const String &config_name, const ScimBridgeDisplay *display, ScimBridgeAgentProtected *new_agent): agent (new_agent), panel_fd (-1), prepared_imcontext_id (-1)
 {
     scim_panel_client = ScimBridgeAgentPanelClient::alloc (config_name, display);
 
-    sem_init (&semaphore, 0, 0);
-}
-
-
-ScimBridgeAgentPanelListenerImpl::~ScimBridgeAgentPanelListenerImpl ()
-{
-    delete scim_panel_client;
-
-    sem_destroy (&semaphore);
-}
-
-
-void ScimBridgeAgentPanelListenerImpl::initialize ()
-{
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_SCIM, 4, "Initializing panel listener...");
-
     scim_panel_client->signal_connect_reload_config (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_reload_config));
     scim_panel_client->signal_connect_exit (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_exit));
     scim_panel_client->signal_connect_update_lookup_table_page_size (slot (this, &ScimBridgeAgentPanelListenerImpl::slot_update_lookup_table_page_size));
@@ -285,200 +171,90 @@ void ScimBridgeAgentPanelListenerImpl::initialize ()
 }
 
 
-retval_t ScimBridgeAgentPanelListenerImpl::launch ()
+ScimBridgeAgentPanelListenerImpl::~ScimBridgeAgentPanelListenerImpl ()
 {
-    assert (!launched);
-    launched = true;
+    delete scim_panel_client;
 
-    thread_status = THREAD_INITIALIZING;
-
-    if (pthread_create (&panel_listener_thread, NULL, &run_panel_listener, this)) {
-        scim_bridge_psyslogln (SYSLOG_CRITICAL, "Cannot invoke the panel listener thread");
-        return RETVAL_FAILED;
-    } else {
-        return RETVAL_SUCCEEDED;
-    }
+    if (panel_fd >= 0) close (panel_fd);
 }
 
 
-void ScimBridgeAgentPanelListenerImpl::panel_listener_launched ()
+retval_t ScimBridgeAgentPanelListenerImpl::initialize ()
 {
-    thread_status = THREAD_INITIALIZED;
-}
-
-
-retval_t ScimBridgeAgentPanelListenerImpl::check_lockfile ()
-{
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 6, "Checking the lockfile...");
-    const char *lockfile_path = scim_bridge_path_get_lockfile ();
-    ifstream input_stream (lockfile_path);
-
-    pid_t pid = 0;
-    input_stream >> pid;
-
-    input_stream.close ();
-
-    if (input_stream.good ()) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 5, "Pinging for the process: pid = %d", pid);
-        if (kill (pid, 0) == 0) {
-            scim_bridge_println ("Another agent is running...");
-            return RETVAL_FAILED;
+    for (int i = 0; i < 10; ++i) {
+        if (open_panel_client ()) {
+            sleep (1);
+        } else {
+            return RETVAL_SUCCEEDED;
         }
     }
-
-    return RETVAL_SUCCEEDED;
+    return RETVAL_FAILED;
 }
 
 
-retval_t ScimBridgeAgentPanelListenerImpl::create_lockfile (pid_t pid)
-{
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 5, "Creating the lockfile...");
-
-    if (update_lockfile (pid)) {
-        return RETVAL_FAILED;
-    } else {
-        return RETVAL_SUCCEEDED;
-    }
-}
-
-
-retval_t ScimBridgeAgentPanelListenerImpl::update_lockfile (pid_t pid)
-{
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 1, "Updating the lockfile...");
-
-    const char *lockfile_path = scim_bridge_path_get_lockfile ();
-    ofstream output_stream (lockfile_path);
-
-    pid_t pid0 = pid;
-    if (pid < 0) pid0 = getpid ();
-
-    output_stream << pid0 << endl;
-    output_stream.close ();
-
-    if (!output_stream.good ()) {
-        scim_bridge_perrorln ("Cannot update the lockfile");
-        return RETVAL_FAILED;
-    } else {
-        return RETVAL_SUCCEEDED;
-    }
-}
-
-
-retval_t ScimBridgeAgentPanelListenerImpl::destroy_lockfile ()
-{
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 5, "Destroying the lockfile...");
-
-    const char *lockfile_path = scim_bridge_path_get_lockfile ();
-    unlink (lockfile_path);
-
-    return RETVAL_SUCCEEDED;
-}
-
-
-void ScimBridgeAgentPanelListenerImpl::panel_listener_exited ()
-{
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 5, "Panel listener exited");
-    thread_status = THREAD_FINALIZED;
-}
-
-
-void ScimBridgeAgentPanelListenerImpl::connect_panel_client ()
+retval_t ScimBridgeAgentPanelListenerImpl::open_panel_client ()
 {
     if (scim_panel_client->open_connection () >= 0) {
         panel_fd = scim_panel_client->get_connection_number ();
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_SCIM, 3, "Panel fd = %d", panel_fd);
+        scim_bridge_pdebugln (3, "Panel fd = %d", panel_fd);
+        return RETVAL_SUCCEEDED;
     } else {
-        scim_bridge_psyslogln (SYSLOG_ERROR, "Failed to open the panel socket");
+        scim_bridge_perrorln ("Failed to open the panel socket");
         panel_fd = -1;
+        return RETVAL_FAILED;
     }
 }
 
 
-void ScimBridgeAgentPanelListenerImpl::panel_error_occured ()
+retval_t ScimBridgeAgentPanelListenerImpl::close_panel_client ()
 {
-    scim_bridge_psyslogln (SYSLOG_WARNING, "Panel client has been downed...");
+    scim_bridge_pdebugln (4, "Close the panel client...");
 
     shutdown (panel_fd, SHUT_RDWR);
     scim_panel_client->close_connection ();
     panel_fd = -1;
     prepared_imcontext_id = -1;
+
+    return RETVAL_FAILED;
 }
 
 
-void ScimBridgeAgentPanelListenerImpl::panel_event_occured ()
-{
-    get_event_server ()->push_event (new ScimBridgeAgentPanelPendingEvent ());
-    sem_wait (&semaphore);
-}
-
-
-void ScimBridgeAgentPanelListenerImpl::read_and_dispatch ()
-{
-    if (panel_fd < 0) {
-        scim_bridge_perrorln ("Connection lost with the panel client");
-        return;
-    }
-
-    fd_set read_set;
-    FD_ZERO (&read_set);
-    FD_SET (panel_fd, &read_set);
-
-    timeval timeout;
-    timeout.tv_sec = 0;
-    timeout.tv_usec = 0;
-
-    if (select (panel_fd + 1, &read_set, NULL, NULL, &timeout) > 0) {
-        if (scim_panel_client->filter_event ()) panel_error_occured ();
-    }
-    sem_post (&semaphore);
-}
-
-
-int ScimBridgeAgentPanelListenerImpl::get_panel_fd ()
+int ScimBridgeAgentPanelListenerImpl::get_socket_fd () const
 {
     return panel_fd;
 }
 
 
-bool ScimBridgeAgentPanelListenerImpl::panel_listener_is_exit_triggered ()
+scim_bridge_agent_event_type_t ScimBridgeAgentPanelListenerImpl::get_trigger_events () const
 {
-    return thread_status != THREAD_INITIALIZING && thread_status != THREAD_INITIALIZED;
+    return SCIM_BRIDGE_AGENT_EVENT_READ | SCIM_BRIDGE_AGENT_EVENT_ERROR;
 }
 
 
-void ScimBridgeAgentPanelListenerImpl::do_close_event_client ()
+bool ScimBridgeAgentPanelListenerImpl::handle_event (scim_bridge_agent_event_type_t event_type)
 {
-    while (thread_status == THREAD_INITIALIZING) usleep (10);
+    scim_bridge_pdebugln (2, "handle_event ()");
 
-    if (thread_status == THREAD_INITIALIZED) {
-        thread_status = THREAD_FINALIZING;
-        shutdown (panel_fd, SHUT_RDWR);
-        scim_panel_client->close_connection ();
-
-        while (thread_status != THREAD_FINALIZED) {
-            scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 3, "Killing panel listener...");
-            sem_post (&semaphore);
-            pthread_kill (panel_listener_thread, SIGNOTIFY);
-            usleep (50);
-        }
-        sem_post (&semaphore);
-        pthread_join (panel_listener_thread, NULL);
+    if ((event_type & SCIM_BRIDGE_AGENT_EVENT_ERROR) || scim_panel_client->filter_event ()) {
+        close_panel_client ();
+        usleep (500);
+        open_panel_client ();
     }
 
-    destroy_lockfile ();
+    return true;
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::prepare (scim_bridge_imcontext_id_t imcontext_id)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_SCIM, 5, "panel prepare");
+    scim_bridge_pdebugln (5, "prepare ()");
     if (panel_fd < 0) {
         scim_bridge_perrorln ("Connection lost with the panel client");
         return;
     }
 
     if (prepared_imcontext_id != -1) {
-        scim_bridge_perrorln ("Panel client has been already prepared");
+        scim_bridge_pdebugln (9, "The panel client has been already prepared");
         return;
     }
 
@@ -489,7 +265,8 @@ void ScimBridgeAgentPanelListenerImpl::prepare (scim_bridge_imcontext_id_t imcon
 
 void ScimBridgeAgentPanelListenerImpl::send ()
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_SCIM, 5, "panel send");
+    scim_bridge_pdebugln (5, "send ()");
+
     if (prepared_imcontext_id == -1) {
         scim_bridge_perrorln ("Panel client has not yet been prepared");
     } else {
@@ -750,7 +527,7 @@ void ScimBridgeAgentPanelListenerImpl::register_input_context (const String &fac
 }
 
 
-void ScimBridgeAgentPanelListenerImpl::remove_input_context ()
+void ScimBridgeAgentPanelListenerImpl::deregister_input_context ()
 {
     if (prepared_imcontext_id == -1) {
         scim_bridge_perrorln ("Panel client has not yet been prepared");
@@ -764,90 +541,158 @@ void ScimBridgeAgentPanelListenerImpl::remove_input_context ()
 /* Slot functions */
 void ScimBridgeAgentPanelListenerImpl::slot_reload_config (scim_bridge_imcontext_id_t imcontext_id)
 {
-    agent->reload_config ();
+    agent->load_config ();
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_exit (scim_bridge_imcontext_id_t imcontext_id)
 {
     scim_bridge_perrorln ("FIXME: not implemented yet: ScimBridgeAgentPanelListenerImpl::slot_exit ()");
-    scim_bridge_psyslogln (SYSLOG_WARNING, "FIXME: not implemented yet: ScimBridgeAgentPanelListenerImpl::slot_exit ()");
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_update_lookup_table_page_size (scim_bridge_imcontext_id_t imcontext_id, int page_size)
 {
-    agent->update_lookup_table_page_size (imcontext_id, page_size);
+    ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
+    if (imcontext != NULL) {
+        prepare (imcontext_id);
+        imcontext->update_lookup_table_page_size (page_size);
+        send ();
+    }
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_lookup_table_page_up (scim_bridge_imcontext_id_t imcontext_id)
 {
-    agent->lookup_table_page_up (imcontext_id);
+    ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
+    if (imcontext != NULL) {
+        prepare (imcontext_id);
+        imcontext->lookup_table_page_up ();
+        send ();
+    }
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_lookup_table_page_down (scim_bridge_imcontext_id_t imcontext_id)
 {
-    agent->lookup_table_page_down (imcontext_id);
+    ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
+    if (imcontext != NULL) {
+        prepare (imcontext_id);
+        imcontext->lookup_table_page_down ();
+        send ();
+    }
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_trigger_property (scim_bridge_imcontext_id_t imcontext_id, const scim::String &property)
 {
-    agent->trigger_property (imcontext_id, property);
+    ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
+    if (imcontext != NULL) {
+        prepare (imcontext_id);
+        imcontext->trigger_property (property);
+        send ();
+    }
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_process_helper_event (scim_bridge_imcontext_id_t imcontext_id, const String &target_uuid, const String &helper_uuid, const Transaction &trans)
 {
-    agent->process_helper_event (imcontext_id, target_uuid, helper_uuid, trans);
+    ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
+    if (imcontext != NULL) {
+        prepare (imcontext_id);
+        imcontext->process_helper_event (target_uuid, helper_uuid, trans);
+        send ();
+    }
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_move_preedit_caret (scim_bridge_imcontext_id_t imcontext_id, int caret_pos)
 {
-    agent->panel_move_preedit_caret (imcontext_id, caret_pos);
+    ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
+    if (imcontext != NULL) {
+        prepare (imcontext_id);
+        imcontext->panel_move_preedit_caret (caret_pos);
+        send ();
+    }
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_select_candidate (scim_bridge_imcontext_id_t imcontext_id, int cand_index)
 {
-    agent->lookup_table_select_candidate (imcontext_id, cand_index);
+    ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
+    if (imcontext != NULL) {
+        prepare (imcontext_id);
+        imcontext->lookup_table_select_candidate (cand_index);
+        send ();
+    }
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_process_key_event (scim_bridge_imcontext_id_t imcontext_id, const KeyEvent &key_event)
 {
-    agent->panel_process_key_event (imcontext_id, key_event);
+    ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
+    if (imcontext != NULL) {
+        prepare (imcontext_id);
+        if (!agent->filter_key_event (imcontext_id, key_event)) {
+            imcontext->forward_key_event (key_event);
+        }
+        send ();
+    }
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_commit (scim_bridge_imcontext_id_t imcontext_id, const WideString &wstr)
 {
-    agent->panel_commit (imcontext_id, wstr);
+    ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
+    if (imcontext != NULL) {
+        prepare (imcontext_id);
+        imcontext->panel_commit_string (wstr);
+        send ();
+    }
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_forward_key_event (scim_bridge_imcontext_id_t imcontext_id, const KeyEvent &key_event)
 {
-    agent->panel_forward_key_event (imcontext_id, key_event);
+    ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
+    if (imcontext != NULL) {
+        prepare (imcontext_id);
+        imcontext->forward_key_event (key_event);
+        send ();
+    }
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_request_help (scim_bridge_imcontext_id_t imcontext_id)
 {
-    agent->panel_request_help (imcontext_id);
+    ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
+    if (imcontext != NULL) {
+        prepare (imcontext_id);
+        imcontext->panel_request_help ();
+        send ();
+    }
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_request_factory_menu (scim_bridge_imcontext_id_t imcontext_id)
 {
-    agent->panel_request_factory_menu (imcontext_id);
+    ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
+    if (imcontext != NULL) {
+        prepare (imcontext_id);
+        agent->request_factory_menu ();
+        send ();
+    }
 }
 
 
 void ScimBridgeAgentPanelListenerImpl::slot_change_factory (scim_bridge_imcontext_id_t imcontext_id, const String &uuid)
 {
-    agent->panel_change_factory (imcontext_id, uuid);
+    ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
+    if (imcontext != NULL) {
+        prepare (imcontext_id);
+        imcontext->panel_change_factory (uuid);
+        send ();
+
+        agent->save_config ();
+    }
 }
