@@ -18,28 +18,49 @@
  * Boston, MA  02111-1307  USA
  */
 
+/**
+ * @file
+ * @author Ryo Dairiki <ryo-dairiki@users.sourceforge.net>
+ * @brief This is the header file for ScimBridgeAgentSignalListener.
+ */
+
 #ifndef SCIMBRIDGEAGENTSIGNALLISTENER_H_
 #define SCIMBRIDGEAGENTSIGNALLISTENER_H_
 
 #include "scim-bridge.h"
-#include "scim-bridge-agent-event-client.h"
+#include "scim-bridge-agent-socket-client.h"
 
-class ScimBridgeAgentSignalListener: public ScimBridgeAgentEventClient
+class ScimBridgeAgentProtected;
+
+/**
+ * The class of signal listeners, which listen to the signals and quit the agent properly.
+ */
+class ScimBridgeAgentSignalListener: public ScimBridgeAgentSocketClient
 {
 
     public:
 
-        static ScimBridgeAgentSignalListener *alloc ();
+        /**
+         * Allocate an signal listener. You cannot allocate more than one signal listener.
+         *
+         * @param agent The agent.
+         * @return A new signal listener, or NULL if it failed to initialize it.
+         */
+        static ScimBridgeAgentSignalListener *alloc (ScimBridgeAgentProtected *agent);
 
-        virtual ~ScimBridgeAgentSignalListener ();
 
-        virtual retval_t launch () = 0;
+        /**
+         * Destructor.
+         */
+        virtual ~ScimBridgeAgentSignalListener () {}
+
 
     protected:
 
-        ScimBridgeAgentSignalListener ();
-
-        virtual void do_close_event_client () = 0;
+        /**
+         * Constructor.
+         */
+        ScimBridgeAgentSignalListener () {}
 
 };
 #endif                                            /*SCIMBRIDGEAGENTSIGNALLISTENER_H_*/
