@@ -29,34 +29,62 @@
  * which use SCIM as a input method to become binary independent with SCIM.\n
  * This will help the maintainers of SCIM, by simplify the complicated dependency around SCIM, libstdc++, and GUI libraries.
  *
+ * <li><a href="../../index.html">The Index of the Documents</a></li>
  * <ul>
- *  <li><a href="user/index.html">User Manual</a></li>
- *  <li><a href="developer/index.html">Developer Manual</a></li>
- *  <li><a href="files.html">The API Document</a></li>
+ *  <li><a href="../../user/index.html">User Manual</a></li>
+ *  <li><a href="../../developer/index.html">Developer Manual</a></li>
  * </ul>
  */
 #ifndef SCIMBRIDGEAGENT_H_
 #define SCIMBRIDGEAGENT_H_
 
 #include "scim-bridge.h"
-#include "scim-bridge-agent-exception.h"
 
-/**/
+/**
+ * The public interface of the agent.
+ */
 class ScimBridgeAgent
 {
 
     public:
 
-        static ScimBridgeAgent *alloc (bool noexit_enabled);
+        /**
+         * Allocate an agent.
+         *
+         * @return A new agent.
+         */
+        static ScimBridgeAgent *alloc ();
 
+        /**
+         * Destructor.
+         */
         virtual ~ScimBridgeAgent () {}
 
-        virtual retval_t create_lockfile (pid_t pid) = 0;
+        /**
+         * Set if it exits automatically when there is no client at all.
+         *
+         * @param enabled Give it true if you want to disable this feature.
+         */
+        virtual void set_noexit_enabled (bool enabled) = 0;
 
-        virtual void launch () throw (ScimBridgeAgentException) = 0;
+        /**
+         * Set if the process daemonize itself when launch () is called.
+         *
+         * @param enabled Give it true if you want to disable this feature.
+         */
+        virtual void set_standalone_enabled (bool enabled) = 0;
+
+        /**
+         * Launch the main loop of the agent.
+         * This funtion blocks until it quits.
+         */
+        virtual retval_t launch () = 0;
 
     protected:
 
+        /**
+         * Constructor.
+         */
         ScimBridgeAgent () {}
 
 };
