@@ -158,10 +158,8 @@ ScimBridgeAgentPanelClient::~ScimBridgeAgentPanelClient ()
 ScimBridgeAgentPanelClientImpl::ScimBridgeAgentPanelClientImpl (const String &new_config_name, const ScimBridgeDisplay *new_display):
 config_name (new_config_name), display (NULL), socket_timeout (scim_get_default_socket_timeout ()), socket_magic_key (0), current_imcontext_id (-1), send_refcount (0)
 {
-    if (new_display != NULL) {
-        display = scim_bridge_alloc_display ();
-        scim_bridge_copy_display (display, new_display);
-    }
+    display = scim_bridge_alloc_display ();
+    scim_bridge_copy_display (display, new_display);
 }
 
 
@@ -173,11 +171,15 @@ ScimBridgeAgentPanelClientImpl::~ScimBridgeAgentPanelClientImpl ()
 
 void ScimBridgeAgentPanelClientImpl::launch_panel () const
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 7, "launch_panel ()");
+    scim_bridge_pdebugln (7, "launch_panel ()");
 
     String panel_program = scim_global_config_read (SCIM_GLOBAL_CONFIG_DEFAULT_PANEL_PROGRAM, String (SCIM_PANEL_PROGRAM));
+
+    if (panel_program.length () <= 0) panel_program = String (SCIM_PANEL_PROGRAM);
+    if (panel_program [0] != SCIM_PATH_DELIM) panel_program = String (SCIM_LIBEXECDIR) + String (SCIM_PATH_DELIM_STRING) + panel_program;
+
     if (access (panel_program.c_str (), X_OK) != 0) panel_program = String (SCIM_PANEL_PROGRAM);
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_AGENT, 5, "Launch \"%s\"", panel_program.c_str ());
+    scim_bridge_pdebugln (5, "Launch \"%s\"", panel_program.c_str ());
 
     if (display != NULL) {
         const pid_t retval = fork ();
