@@ -55,9 +55,9 @@ void scim_bridge_perrorln (const char *string,...)
 }
 
 
-void scim_bridge_pdebugln (scim_bridge_debug_flag_t flag, scim_bridge_debug_level_t level, const char *string,...)
+void scim_bridge_pdebugln (scim_bridge_debug_level_t level, const char *string,...)
 {
-    if ((flag & scim_bridge_debug_get_flags ()) && ((10 - level) <= scim_bridge_debug_get_level ())) {
+    if ((10 - level) <= scim_bridge_debug_get_level ()) {
         va_list ap;
         va_start (ap, string);
         const size_t str_length = strlen (string);
@@ -89,9 +89,9 @@ void scim_bridge_perror (const char *string,...)
 }
 
 
-void scim_bridge_pdebug (scim_bridge_debug_flag_t flag, scim_bridge_debug_level_t level, const char *string,...)
+void scim_bridge_pdebug (scim_bridge_debug_level_t level, const char *string,...)
 {
-    if ((flag & scim_bridge_debug_get_flags ()) && ((10 - level) <= scim_bridge_debug_get_level ())) {
+    if ((10 - level) <= scim_bridge_debug_get_level ()) {
         va_list ap;
         va_start (ap, string);
         vfprintf (stdout, string, ap);
