@@ -18,6 +18,12 @@
  * Boston, MA  02111-1307  USA
  */
 
+/**
+ * @file
+ * @author Ryo Dairiki <ryo-dairiki@users.sourceforge.net>
+ * @brief This is the header for the protected interfaces of the clients.
+ */
+
 #ifndef SCIMBRIDGECLIENTPROTECTED_H_
 #define SCIMBRIDGECLIENTPROTECTED_H_
 
@@ -26,7 +32,15 @@ extern "C"
 {
 #endif
 
-    void scim_bridge_client_connection_lost ();
+    /**
+     * The connection is established with the agent.
+     */
+    void scim_bridge_client_messenger_opened ();
+
+    /**
+     * The connection with the agent is closed.
+     */
+    void scim_bridge_client_messenger_closed ();
 
 #ifdef __cplusplus
 }
