@@ -19,6 +19,7 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include <gtk/gtk.h>
@@ -54,11 +55,16 @@ void im_module_exit (void);
 void im_module_list (const GtkIMContextInfo ***contexts, int *context_count);
 GtkIMContext *im_module_create (const gchar *context_id);
 
+
 /* Implementations */
 void im_module_init (GTypeModule *type_module)
 {
     scim_bridge_client_imcontext_register_type (type_module);
     scim_bridge_client_gtk_initialize ();
+    
+    static boolean first_time = TRUE;
+    if (first_time) atexit (scim_bridge_client_gtk_finalize);
+    first_time = FALSE;
 }
 
 
