@@ -21,6 +21,7 @@
 #include <alloca.h>
 #include <assert.h>
 #include <errno.h>
+#include <malloc.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -32,7 +33,6 @@
 #include "scim-bridge-client.h"
 #include "scim-bridge-client-imcontext-gtk.h"
 #include "scim-bridge-client-key-event-utility-gtk.h"
-#include "scim-bridge-exception.h"
 #include "scim-bridge-imcontext.h"
 #include "scim-bridge-messenger.h"
 #include "scim-bridge-output.h"
@@ -43,9 +43,6 @@ struct _ScimBridgeClientIMContext
 {
     GtkIMContext parent;
 
-    ScimBridgeClientIMContext *prev;
-    ScimBridgeClientIMContext *next;
-
     scim_bridge_imcontext_id_t id;
 
     char *preedit_string;
@@ -67,15 +64,9 @@ struct _ScimBridgeClientIMContext
     int cursor_y;
     int window_x;
     int window_y;
-
-    unsigned int flags;
 };
 
 /* Private variables */
-static const unsigned int FLAG_NONE = 0;
-static const unsigned int FLAG_PENDING = 1 << 0;
-static const unsigned int FLAG_RESET = 1 << 1;
-
 static GdkColor preedit_normal_background;
 static GdkColor preedit_normal_foreground;
 static GdkColor preedit_active_background;
@@ -84,8 +75,6 @@ static GdkColor preedit_active_foreground;
 static GType class_type = 0;
 static GObjectClass *root_klass = NULL;
 
-static ScimBridgeClientIMContext *first_imcontext = NULL;
-static ScimBridgeClientIMContext *last_imcontext = NULL;
 static ScimBridgeClientIMContext *focused_imcontext = NULL;
 
 static GtkIMContext *fallback_imcontext = NULL;
@@ -93,7 +82,7 @@ static gulong  fallback_commit_handler;
 
 static guint key_snooper_id = 0;
 static boolean key_snooper_used = FALSE;
-static boolean key_snooper_enabled = FALSE;
+
 
 /* Class functions */
 static void scim_bridge_client_imcontext_class_initialize (ScimBridgeClientIMContextClass *klass, gpointer *klass_data);
@@ -111,62 +100,92 @@ static void scim_bridge_client_imcontext_focus_in (GtkIMContext *context);
 static void scim_bridge_client_imcontext_focus_out (GtkIMContext *context);
 static void scim_bridge_client_imcontext_set_cursor_location (GtkIMContext *context, GdkRectangle *area);
 
+
 /* Helper functions */
-static void connection_lost ()
+static void fallback_commit (GtkIMContext *context, const char *str, gpointer data)
 {
-    ScimBridgeClientIMContext *seek;
-    for (seek = first_imcontext; seek != NULL; seek = seek->next) {
-        scim_bridge_client_imcontext_set_preedit_shown (seek, FALSE);
-        seek->id = -1;
-    }
+    scim_bridge_pdebugln (4, "fallback_commit ()");
+    if (str != NULL && focused_imcontext != NULL) g_signal_emit_by_name (focused_imcontext, "commit", str);
 }
 
 
-static void fallback_commit (GtkIMContext *context, const char *str, gpointer data)
+static retval_t filter_key_event (ScimBridgeClientIMContext *imcontext, GdkEventKey *event, boolean *consumed)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 4, "fallback_commit");
-    if (str != NULL && focused_imcontext != NULL) g_signal_emit_by_name (focused_imcontext, "commit", str);
+    scim_bridge_pdebugln (5, "filter_key_event ()");
+    
+    if (focused_imcontext != imcontext) scim_bridge_client_imcontext_focus_in (GTK_IM_CONTEXT (imcontext));
+
+    if (scim_bridge_client_is_messenger_opened ()) {
+        ScimBridgeKeyEvent *bridge_key_event = scim_bridge_alloc_key_event ();
+        scim_bridge_key_event_gdk_to_bridge (bridge_key_event, imcontext->client_window, event);
+
+        *consumed = FALSE;
+        const retval_t retval_error = scim_bridge_client_handle_key_event (imcontext, bridge_key_event, consumed);
+        scim_bridge_free_key_event (bridge_key_event);
+
+        if (retval_error) {
+            scim_bridge_perrorln ("An IOException at filter_key_event ()");
+        } else {
+            return RETVAL_SUCCEEDED;
+        }
+    }
+
+    return RETVAL_FAILED;
+}
+
+
+static retval_t set_cursor_location (ScimBridgeClientIMContext *imcontext, int window_x, int window_y, int cursor_x, int cursor_y)
+{
+    scim_bridge_pdebugln (5, "update_cursor_location ()");
+
+    if (imcontext->window_x == window_x && imcontext->window_y == window_y && imcontext->cursor_x == cursor_x && imcontext->cursor_y == cursor_y) {
+        return RETVAL_SUCCEEDED;
+    } else {
+        imcontext->cursor_x = cursor_x;
+        imcontext->cursor_y = cursor_y;
+        imcontext->window_x = window_x;
+        imcontext->window_y = window_y;
+
+        scim_bridge_pdebugln (3, "The cursor location is changed: x = %d + %d\ty = %d + %d", imcontext->window_x, imcontext->cursor_x, imcontext->window_y, imcontext->cursor_y);
+
+        if (scim_bridge_client_is_messenger_opened ()) {
+            if (scim_bridge_client_set_cursor_location (imcontext, imcontext->window_x + imcontext->cursor_x, imcontext->window_y + imcontext->cursor_y)) {
+                scim_bridge_perrorln ("An IOException occurred at set_cursor_location ()");
+                return RETVAL_FAILED;
+            } else {
+                return RETVAL_SUCCEEDED;
+            }
+        }
+    }
+
+    return RETVAL_FAILED;
 }
 
 
 static gboolean key_snooper (GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 7, "key_snooper");
+    scim_bridge_pdebugln (7, "key_snooper ()");
 
-    if (scim_bridge_client_is_active () && focused_imcontext != NULL) {
+    if (scim_bridge_client_is_messenger_opened () && focused_imcontext != NULL) {
         if (focused_imcontext->client_window != NULL) {
             int new_window_x;
             int new_window_y;
             gdk_window_get_origin (focused_imcontext->client_window, &new_window_x, &new_window_y);
 
             if (focused_imcontext->window_x != new_window_x || focused_imcontext->window_y != new_window_y) {
-                focused_imcontext->window_x = new_window_x;
-                focused_imcontext->window_y = new_window_y;
-
-                scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 1,
-                    "The cursor location is changed: x = %d + %d\ty = %d + %d",
-                    focused_imcontext->window_x, focused_imcontext->cursor_x, focused_imcontext->window_y, focused_imcontext->cursor_y);
-
-                if (scim_bridge_client_cursor_location_changed (focused_imcontext, focused_imcontext->window_x + focused_imcontext->cursor_x, focused_imcontext->window_y + focused_imcontext->cursor_y)) {
-                    scim_bridge_exception_push_stack ("key_snooper ()");
-                    scim_bridge_exception_output ();
-                    connection_lost ();
+                if (set_cursor_location (focused_imcontext, new_window_x, new_window_y, focused_imcontext->cursor_x, focused_imcontext->cursor_y)) {
+                    scim_bridge_perrorln ("An IOException at key_snooper ()");
+                    return FALSE;
                 }
             }
         }
 
-        ScimBridgeKeyEvent *bridge_key_event = scim_bridge_alloc_key_event ();
-        scim_bridge_key_event_gdk_to_bridge (bridge_key_event, focused_imcontext->client_window, event);
-
         boolean consumed = FALSE;
-        const retval_t retval_error = scim_bridge_client_key_event_occured (focused_imcontext, bridge_key_event, &consumed);
-
-        if (retval_error) {
-            scim_bridge_exception_push_stack ("key_snooper ()");
-            scim_bridge_exception_output ();
-            connection_lost ();
-        } else if (consumed) {
-            return TRUE;
+        if (filter_key_event (focused_imcontext, event, &consumed)) {
+            scim_bridge_perrorln ("An IOException at key_snooper ()");
+            return FALSE;
+        } else {
+            if (consumed) return TRUE;
         }
     }
 
@@ -176,12 +195,13 @@ static gboolean key_snooper (GtkWidget *widget, GdkEventKey *event, gpointer dat
 
 static boolean is_key_snooper_enabled ()
 {
-    static boolean first_run = TRUE;
+    static boolean first_time = TRUE;
+    static boolean key_snooper_enabled = FALSE;
 
-    if (first_run) {
+    if (first_time) {
         char *env_key_snooper_enabled = getenv ("SCIM_BRIDGE_KEY_SNOOPER_ENABLED");
         if (env_key_snooper_enabled != NULL) scim_bridge_string_to_boolean (&key_snooper_enabled, env_key_snooper_enabled);
-        first_run = FALSE;
+        first_time = FALSE;
     }
 
     return key_snooper_enabled;
@@ -189,21 +209,6 @@ static boolean is_key_snooper_enabled ()
 
 
 /* Bindings */
-ScimBridgeClientIMContext *scim_bridge_client_find_imcontext (scim_bridge_imcontext_id_t id)
-{
-    if (focused_imcontext != NULL && focused_imcontext->id == id) {
-        return focused_imcontext;
-    } else {
-        ScimBridgeClientIMContext *seek;
-        for (seek = first_imcontext; seek != NULL; seek = seek->next) {
-            if (seek->id == id) return seek;
-        }
-
-        return NULL;
-    }
-}
-
-
 void scim_bridge_client_imcontext_set_id (ScimBridgeClientIMContext *imcontext, scim_bridge_imcontext_id_t new_id)
 {
     imcontext->id = new_id;
@@ -277,7 +282,7 @@ void scim_bridge_client_imcontext_set_preedit_attributes (ScimBridgeClientIMCont
 }
 
 
-void scim_bridge_client_imcontext_update_preedit (const ScimBridgeClientIMContext *imcontext)
+void scim_bridge_client_imcontext_update_preedit (ScimBridgeClientIMContext *imcontext)
 {
     g_signal_emit_by_name ((ScimBridgeClientIMContext*) imcontext, "preedit_changed");
 }
@@ -291,6 +296,7 @@ void scim_bridge_client_imcontext_set_commit_string (ScimBridgeClientIMContext *
     } else {
         commit_string_length = 0;
     }
+    
     if (imcontext->commit_string_capacity <= commit_string_length) {
         imcontext->commit_string_capacity = commit_string_length;
         free (imcontext->commit_string);
@@ -304,28 +310,24 @@ void scim_bridge_client_imcontext_set_commit_string (ScimBridgeClientIMContext *
 }
 
 
-void scim_bridge_client_imcontext_commit (const ScimBridgeClientIMContext *imcontext)
+void scim_bridge_client_imcontext_commit (ScimBridgeClientIMContext *imcontext)
 {
     g_signal_emit_by_name ((ScimBridgeClientIMContext*) imcontext, "commit", imcontext->commit_string);
 }
 
 
-void scim_bridge_client_imcontext_beep (const ScimBridgeClientIMContext *imcontext)
+void scim_bridge_client_imcontext_beep (ScimBridgeClientIMContext *imcontext)
 {
     gdk_beep ();
 }
 
 
-boolean scim_bridge_client_imcontext_get_surrounding_text (const ScimBridgeClientIMContext *imcontext, int before_max, int after_max, char **string, int *cursor_position)
+boolean scim_bridge_client_imcontext_get_surrounding_text (ScimBridgeClientIMContext *imcontext, int before_max, int after_max, char **string, int *cursor_position)
 {
     char *str;
     int cur_pos_in_utf8;
 
     if (gtk_im_context_get_surrounding (GTK_IM_CONTEXT (imcontext), &str, &cur_pos_in_utf8)) {
-        const size_t fetch_str_length = strlen (str);
-        const size_t after_str_length = fetch_str_length - cur_pos_in_utf8;
-        const size_t before_str_length = fetch_str_length - after_str_length;
-
         const size_t fetch_wstr_length = g_utf8_strlen (str, -1);
         const size_t after_wstr_length = g_utf8_strlen (str + cur_pos_in_utf8, -1);
         const size_t before_wstr_length = fetch_wstr_length - after_wstr_length;
@@ -365,21 +367,21 @@ boolean scim_bridge_client_imcontext_get_surrounding_text (const ScimBridgeClien
 }
 
 
-boolean scim_bridge_client_imcontext_delete_surrounding_text (ScimBridgeClientIMContext *imcontext, int offset, int length, boolean *need_reset)
+boolean scim_bridge_client_imcontext_delete_surrounding_text (ScimBridgeClientIMContext *imcontext, int offset, int length)
 {
-    imcontext->flags = FLAG_PENDING;
     boolean retval = gtk_im_context_delete_surrounding (GTK_IM_CONTEXT (imcontext), offset, length);
-    if (imcontext->flags & FLAG_RESET) {
-        *need_reset = TRUE;
-    } else {
-        *need_reset = FALSE;
-    }
-    imcontext->flags = FLAG_NONE;
     return retval;
 }
 
 
-void scim_bridge_client_imcontext_forward_key_event (const ScimBridgeClientIMContext *imcontext, const ScimBridgeKeyEvent *key_event)
+boolean scim_bridge_client_imcontext_replace_surrounding_text (ScimBridgeClientIMContext *imcontext, int cursor_position, const char *string)
+{
+    gtk_im_context_set_surrounding (GTK_IM_CONTEXT (imcontext), string, -1, cursor_position);
+    return TRUE;
+}
+
+
+void scim_bridge_client_imcontext_forward_key_event (ScimBridgeClientIMContext *imcontext, const ScimBridgeKeyEvent *key_event)
 {
     GdkEventKey gdk_event;
     scim_bridge_key_event_bridge_to_gdk (&gdk_event, imcontext->client_window, key_event);
@@ -395,8 +397,6 @@ void scim_bridge_client_imcontext_static_initialize ()
     gdk_color_parse ("light blue", &preedit_active_background);
     gdk_color_parse ("black", &preedit_active_foreground);
 
-    first_imcontext = NULL;
-    last_imcontext = NULL;
     focused_imcontext = NULL;
 
     fallback_imcontext = gtk_im_context_simple_new ();
@@ -413,7 +413,20 @@ void scim_bridge_client_imcontext_static_finalize ()
         key_snooper_id = 0;
         key_snooper_used = FALSE;
     }
+
     fallback_imcontext = NULL;
+    focused_imcontext = NULL;
+}
+
+
+void scim_bridge_client_imcontext_connection_opened ()
+{
+}
+
+
+void scim_bridge_client_imcontext_connection_closed ()
+{
+    if (focused_imcontext != NULL) scim_bridge_client_imcontext_set_preedit_shown (focused_imcontext, FALSE);
 }
 
 
@@ -444,6 +457,8 @@ GType scim_bridge_client_imcontext_get_type ()
 
 void scim_bridge_client_imcontext_register_type (GTypeModule *type_module)
 {
+	scim_bridge_pdebugln (2, "scim_bridge_client_imcontext_register_type ()");
+
     static const GTypeInfo klass_info = {
         sizeof (ScimBridgeClientIMContextClass),
         /* no base class initializer */
@@ -468,6 +483,8 @@ void scim_bridge_client_imcontext_register_type (GTypeModule *type_module)
 
 GtkIMContext *scim_bridge_client_imcontext_new ()
 {
+    scim_bridge_pdebugln (4, "scim_bridge_client_imcontext_new ()");
+
     ScimBridgeClientIMContext *ic = SCIM_BRIDGE_CLIENT_IMCONTEXT (g_object_new (GTK_TYPE_SCIM_CLIENT_IMCONTEXT, NULL));
     return GTK_IM_CONTEXT (ic);
 }
@@ -475,7 +492,7 @@ GtkIMContext *scim_bridge_client_imcontext_new ()
 
 void scim_bridge_client_imcontext_initialize (ScimBridgeClientIMContext *imcontext, ScimBridgeClientIMContextClass *klass)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 5, "scim_bridge_client_imcontext_initialize");
+    scim_bridge_pdebugln (5, "scim_bridge_client_imcontext_initialize  ()");
 
     imcontext->preedit_string = malloc (sizeof (char));
     imcontext->preedit_string[0] = '\0';
@@ -491,49 +508,32 @@ void scim_bridge_client_imcontext_initialize (ScimBridgeClientIMContext *imconte
 
     imcontext->client_window = NULL;
 
-    imcontext->flags = FLAG_NONE;
+    imcontext->id = -1;
 
-    if (!scim_bridge_client_is_active ()) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_CLIENT, 4, "The messenger is now down");
-        imcontext->id = -1;
-    } else if (scim_bridge_client_alloc_imcontext (imcontext)) {
-        scim_bridge_exception_push_stack ("scim_bridge_client_imcontext_initialize ()");
-        scim_bridge_exception_output ();
-        connection_lost ();
-        imcontext->id = -1;
+    if (!scim_bridge_client_is_messenger_opened ()) {
+        scim_bridge_perrorln ("The messenger is now down");
+    } else if (scim_bridge_client_register_imcontext (imcontext)) {
+        scim_bridge_perrorln ("Failed to register the IMContext");
     } else {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 1, "IMContext allocated: id = %d", imcontext->id);
-    }
-
-    if (last_imcontext == NULL) {
-        imcontext->prev = NULL;
-        imcontext->next = NULL;
-        first_imcontext = imcontext;
-        last_imcontext = imcontext;
-    } else {
-        last_imcontext->next = imcontext;
-        imcontext->prev = last_imcontext;
-        last_imcontext = imcontext;
+        scim_bridge_pdebugln (1, "IMContext registered: id = %d", imcontext->id);
     }
 }
 
 
 void scim_bridge_client_imcontext_finalize (GObject *object)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 5, "scim_bridge_client_imcontext_finalize");
+    scim_bridge_pdebugln (5, "scim_bridge_client_imcontext_finalize ()");
 
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (object);
 
     if (imcontext == focused_imcontext) scim_bridge_client_imcontext_focus_out (GTK_IM_CONTEXT (imcontext));
 
-    if (!scim_bridge_client_is_active ()) {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_CLIENT, 4, "The messenger is now down");
-    } else if (scim_bridge_client_free_imcontext (imcontext)) {
-        scim_bridge_exception_push_stack ("scim_bridge_client_imcontext_finitialize ()");
-        scim_bridge_exception_output ();
-        connection_lost ();
+    if (!scim_bridge_client_is_messenger_opened ()) {
+        scim_bridge_perrorln ("The messenger is now down");
+    } else if (scim_bridge_client_deregister_imcontext (imcontext)) {
+        scim_bridge_perrorln ("Failed to deregister an IMContext");
     } else {
-        scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 3, "IMContext freed: id = %d", imcontext->id);
+        scim_bridge_pdebugln (3, "IMContext deregistered: id = %d", imcontext->id);
     }
 
     if (imcontext->client_window) g_object_unref (imcontext->client_window);
@@ -550,15 +550,6 @@ void scim_bridge_client_imcontext_finalize (GObject *object)
     imcontext->preedit_attribute_count = 0;
     imcontext->preedit_attribute_capacity = 0;
 
-    ScimBridgeClientIMContext *prev = imcontext->prev;
-    ScimBridgeClientIMContext *next = imcontext->next;
-
-    if (prev != NULL) prev->next = next;
-    if (next != NULL) next->prev = prev;
-
-    if (imcontext == first_imcontext) first_imcontext = next;
-    if (imcontext == last_imcontext) last_imcontext = prev;
-
     root_klass->finalize (object);
 }
 
@@ -566,12 +557,11 @@ void scim_bridge_client_imcontext_finalize (GObject *object)
 /* Class functions */
 gboolean scim_bridge_client_imcontext_filter_key_event (GtkIMContext *context, GdkEventKey *event)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 8, "scim_bridge_client_imcontext_filter_key_event");
+    scim_bridge_pdebugln (8, "scim_bridge_client_imcontext_filter_key_event ()");
 
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (context);
 
-    if (scim_bridge_client_is_active () && imcontext != NULL && !key_snooper_used) {
-        if (focused_imcontext != imcontext) scim_bridge_client_imcontext_focus_in (context);
+    if (scim_bridge_client_is_messenger_opened () && imcontext != NULL && !key_snooper_used) {
 
         if (imcontext->client_window != NULL) {
             int new_window_x;
@@ -582,53 +572,40 @@ gboolean scim_bridge_client_imcontext_filter_key_event (GtkIMContext *context, G
                 imcontext->window_x = new_window_x;
                 imcontext->window_y = new_window_y;
 
-                scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 1,
+                scim_bridge_pdebugln (1,
                     "The cursor location is changed: x = %d + %d\ty = %d + %d",
                     imcontext->window_x, imcontext->cursor_x, imcontext->window_y, imcontext->cursor_y);
 
-                if (scim_bridge_client_cursor_location_changed (imcontext, imcontext->window_x + imcontext->cursor_x, imcontext->window_y + imcontext->cursor_y)) {
-                    scim_bridge_exception_push_stack ("scim_bridge_client_imcontext_filter_key_event ()");
-                    scim_bridge_exception_output ();
-                    connection_lost ();
+                if (set_cursor_location (imcontext, new_window_x, new_window_y, imcontext->cursor_x, imcontext->cursor_y)) {
+                    scim_bridge_perrorln ("An IOException occurred at scim_bridge_client_imcontext_filter_key_event ()");
+                    return gtk_im_context_filter_keypress (fallback_imcontext, event);
                 }
             }
         }
 
-        ScimBridgeKeyEvent *bridge_key_event = scim_bridge_alloc_key_event ();
-        scim_bridge_key_event_gdk_to_bridge (bridge_key_event, imcontext->client_window, event);
-
         boolean consumed = FALSE;
-        const retval_t retval_error = scim_bridge_client_key_event_occured (imcontext, bridge_key_event, &consumed);
-
-        if (retval_error) {
-            scim_bridge_exception_push_stack ("scim_bridge_client_imcontext_filter_key_event ()");
-            scim_bridge_exception_output ();
-            connection_lost ();
+        if (filter_key_event (imcontext, event, &consumed)) {
+            scim_bridge_perrorln ("An IOException occurred at scim_bridge_client_imcontext_filter_key_event ()");
         } else if (consumed) {
             return TRUE;
         }
     }
 
-    // TODO Press F11 to reconnect with the agent
     return gtk_im_context_filter_keypress (fallback_imcontext, event);
 }
 
 
 void scim_bridge_client_imcontext_reset (GtkIMContext *context)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 8, "scim_bridge_client_imcontext_reset");
+    scim_bridge_pdebugln (8, "scim_bridge_client_imcontext_reset ()");
 
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (context);
 
     if (imcontext != focused_imcontext) return;
 
-    if (scim_bridge_client_is_active () && imcontext != NULL) {
-        if (imcontext->flags & FLAG_PENDING) {
-            imcontext->flags |= FLAG_RESET;
-        } else if (scim_bridge_client_reset_imcontext (imcontext)) {
-            scim_bridge_exception_push_stack ("scim_bridge_client_imcontext_reset ()");
-            scim_bridge_exception_output ();
-            connection_lost ();
+    if (scim_bridge_client_is_messenger_opened () && imcontext != NULL) {
+        if (scim_bridge_client_reset_imcontext (imcontext)) {
+            scim_bridge_perrorln ("An IOException occurred at scim_bridge_client_imcontext_reset ()");
         }
     }
 }
@@ -636,11 +613,11 @@ void scim_bridge_client_imcontext_reset (GtkIMContext *context)
 
 void scim_bridge_client_imcontext_get_preedit_string (GtkIMContext *context, gchar **str, PangoAttrList **pango_attrs, gint *cursor_pos)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 8, "scim_bridge_client_imcontext_get_preedit_string");
+    scim_bridge_pdebugln (4, "scim_bridge_client_imcontext_get_preedit_string ()");
 
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (context);
 
-    if (scim_bridge_client_is_active () && imcontext != NULL && imcontext->preedit_shown) {
+    if (scim_bridge_client_is_messenger_opened () && imcontext != NULL && imcontext->preedit_shown) {
         const size_t preedit_string_length = strlen (imcontext->preedit_string);
         
         if (str) {
@@ -746,7 +723,7 @@ void scim_bridge_client_imcontext_get_preedit_string (GtkIMContext *context, gch
 
 void scim_bridge_client_imcontext_focus_in (GtkIMContext *context)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 8,"scim_bridge_client_imcontext_focus_in");
+    scim_bridge_pdebugln (8, "scim_bridge_client_imcontext_focus_in ()");
 
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (context);
 
@@ -754,15 +731,18 @@ void scim_bridge_client_imcontext_focus_in (GtkIMContext *context)
 
     focused_imcontext = imcontext;
 
+    if (!scim_bridge_client_is_messenger_opened () && scim_bridge_client_is_reconnection_enabled ()) {
+        scim_bridge_pdebugln (7, "Trying to open the connection again...");
+        scim_bridge_client_open_messenger ();
+    }
+
     if (is_key_snooper_enabled ()) {
         key_snooper_id = gtk_key_snooper_install ((GtkKeySnoopFunc) &key_snooper, NULL);
         key_snooper_used = TRUE;
     }
-    if (scim_bridge_client_is_active () && imcontext != NULL) {
-        if (scim_bridge_client_focus_changed (imcontext, TRUE)) {
-            scim_bridge_exception_push_stack ("scim_bridge_client_imcontext_focus_in ()");
-            scim_bridge_exception_output ();
-            connection_lost ();
+    if (scim_bridge_client_is_messenger_opened () && imcontext != NULL) {
+        if (scim_bridge_client_change_focus (imcontext, TRUE)) {
+            scim_bridge_perrorln ("An IOException occurred at scim_bridge_client_imcontext_focus_in ()");
         }
     }
 }
@@ -770,7 +750,7 @@ void scim_bridge_client_imcontext_focus_in (GtkIMContext *context)
 
 void scim_bridge_client_imcontext_focus_out (GtkIMContext *context)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 8, "scim_bridge_client_imcontext_focus_out");
+    scim_bridge_pdebugln (8, "scim_bridge_client_imcontext_focus_out ()");
 
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (context);
 
@@ -779,12 +759,10 @@ void scim_bridge_client_imcontext_focus_out (GtkIMContext *context)
         scim_bridge_client_imcontext_set_preedit_shown (imcontext, FALSE);
         scim_bridge_client_imcontext_update_preedit (imcontext);
     }
-    if (scim_bridge_client_is_active () && imcontext != NULL) {
+    if (scim_bridge_client_is_messenger_opened () && imcontext != NULL) {
 
-        if (scim_bridge_client_focus_changed (imcontext, FALSE)) {
-            scim_bridge_exception_push_stack ("scim_bridge_client_imcontext_focus_out ()");
-            scim_bridge_exception_output ();
-            connection_lost ();
+        if (scim_bridge_client_change_focus (imcontext, FALSE)) {
+            scim_bridge_perrorln ("An IOException occurred at scim_bridge_client_imcontext_focus_out ()");
         }
     }
     if (key_snooper_used) {
@@ -792,28 +770,31 @@ void scim_bridge_client_imcontext_focus_out (GtkIMContext *context)
         key_snooper_id = 0;
         key_snooper_used = FALSE;
     }
+
     focused_imcontext = NULL;
 }
 
 
 void scim_bridge_client_imcontext_set_client_window (GtkIMContext *context, GdkWindow *new_window)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 8, "scim_bridge_client_imcontext_set_client_window");
+    scim_bridge_pdebugln (7, "scim_bridge_client_imcontext_set_client_window ()");
 
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (context);
 
     if (imcontext != NULL) {
         if (imcontext->client_window != NULL) g_object_unref (imcontext->client_window);
-        if (new_window != NULL) g_object_ref (new_window);
         imcontext->client_window = new_window;
-        gdk_window_get_origin (imcontext->client_window, &imcontext->window_x, &imcontext->window_y);
+        if (new_window != NULL) {
+            g_object_ref (new_window);
+            gdk_window_get_origin (imcontext->client_window, &imcontext->window_x, &imcontext->window_y);
+        }
     }
 }
 
 
 void scim_bridge_client_imcontext_set_cursor_location (GtkIMContext *context, GdkRectangle *area)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 8, "scim_bridge_client_imcontext_set_cursor_location");
+    scim_bridge_pdebugln (4, "scim_bridge_client_imcontext_set_cursor_location ()");
 
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (context);
 
@@ -825,22 +806,8 @@ void scim_bridge_client_imcontext_set_cursor_location (GtkIMContext *context, Gd
         int new_window_y;
         gdk_window_get_origin (imcontext->client_window, &new_window_x, &new_window_y);
 
-        if (imcontext->window_x + imcontext->cursor_x != new_window_x + new_cursor_x || imcontext->window_y + imcontext->cursor_y != new_cursor_y + new_window_y) {
-            imcontext->window_x = new_window_x;
-            imcontext->window_y = new_window_y;
-            imcontext->cursor_x = new_cursor_x;
-            imcontext->cursor_y = new_cursor_y;
-
-            scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 1, "The cursor location is changed: x = %d + %d\ty = %d + %d",
-                imcontext->window_x, imcontext->cursor_x, imcontext->window_y, imcontext->cursor_y);
-
-            if (scim_bridge_client_is_active ()) {
-                if (scim_bridge_client_cursor_location_changed (imcontext, imcontext->window_x + imcontext->cursor_x, imcontext->window_y + imcontext->cursor_y)) {
-                    scim_bridge_exception_push_stack ("scim_bridge_client_imcontext_set_cursor_location ()");
-                    scim_bridge_exception_output ();
-                    connection_lost ();
-                }
-            }
+        if (set_cursor_location (imcontext, new_window_x, new_window_y, new_cursor_x, new_cursor_y)) {
+            scim_bridge_perrorln ("An IOException occurred at scim_bridge_client_imcontext_set_cursor_location ()");
         }
     }
 }
@@ -848,15 +815,15 @@ void scim_bridge_client_imcontext_set_cursor_location (GtkIMContext *context, Gd
 
 void scim_bridge_client_imcontext_set_preedit_enabled (GtkIMContext *context, gboolean enabled)
 {
-    scim_bridge_pdebugln (SCIM_BRIDGE_DEBUG_IMCONTEXT, 8, "scim_bridge_client_imcontext_set_preedit_enabled");
+    scim_bridge_pdebugln (8, "scim_bridge_client_imcontext_set_preedit_enabled ()");
 
     ScimBridgeClientIMContext *imcontext = SCIM_BRIDGE_CLIENT_IMCONTEXT (context);
 
-    if (scim_bridge_client_is_active () && imcontext != NULL) {
-        if (scim_bridge_client_set_preedit_enabled (imcontext, enabled)) {
-            scim_bridge_exception_push_stack ("scim_bridge_client_imcontext_set_preedit_enabled ()");
-            scim_bridge_exception_output ();
-            connection_lost ();
+    if (imcontext != NULL) {
+        if (scim_bridge_client_is_messenger_opened ()) {
+            if (scim_bridge_client_set_preedit_mode (imcontext, enabled ? PREEDIT_EMBEDDED:PREEDIT_ANY)) {
+                scim_bridge_perrorln ("An IOException occurred at scim_bridge_client_imcontext_set_preedit_enabled ()");
+            }
         }
     }
 }
