@@ -18,6 +18,13 @@
  * Boston, MA  02111-1307  USA
  */
 
+/**
+ * @file
+ * @author Ryo Dairiki <ryo-dairiki@users.sourceforge.net>
+ * @brief This is the header for gtk imcontext of scim-bridge.
+ */
+
+
 #ifndef SCIMBRIDGECLIENTIMCONTEXTGTK_H_
 #define SCIMBRIDGECLIENTIMCONTEXTGTK_H_
 
@@ -36,16 +43,55 @@
 #define IS_SCIM_BRIDGE_CLIENT_IMCONTEXT_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_SCIM_CLIENT_IMCONTEXT))
 #define SCIM_BRIDGE_CLIENT_IMCONTEXT_GET_CLASS(obj) (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_SCIM_CLIENT_IMCONTEXT, ScimBridgeClientIMContextClass))
 
-typedef struct _ScimBridgeClientIMContextClass
+struct _ScimBridgeClientIMContextClass
 {
     GtkIMContextClass parent_class;
-} ScimBridgeClientIMContextClass;
+};
 
+/**
+ * IMContext class for gtk client.
+ */
+typedef struct _ScimBridgeClientIMContextClass ScimBridgeClientIMContextClass;
+
+/**
+ * Initialize IMContext class itself.
+ */
 void scim_bridge_client_imcontext_static_initialize ();
+
+/**
+ * Finalize IMContext class itself.
+ */
 void scim_bridge_client_imcontext_static_finalize ();
 
+/**
+ * The connection with the agent is opened.
+ */
+void scim_bridge_client_imcontext_connection_opened ();
+
+/**
+ * The connection with the agent is closed.
+ */
+void scim_bridge_client_imcontext_connection_closed ();
+
+/**
+ * Allocate an IMContext.
+ *
+ * @return new IMContext.
+ */
 GtkIMContext *scim_bridge_client_imcontext_new ();
 
+/**
+ * Get the type value of IMContexts.
+ *
+ * @return The type value of IMContexts.
+ */
 GType scim_bridge_client_imcontext_get_type ();
+
+/**
+ * Register the type value for IMContexts.
+ *
+ * @param type_module The type module.
+ */
 void scim_bridge_client_imcontext_register_type (GTypeModule *type_module);
+
 #endif                                            /*SCIMBRIDGECLIENTIMCONTEXTGTK_H_*/
