@@ -18,6 +18,8 @@
  * Boston, MA  02111-1307  USA
  */
 
+#include <stdint.h>
+
 /**
  * @file
  * @author Ryo Dairiki <ryo-dairiki@users.sourceforge.net>
@@ -26,6 +28,19 @@
 #ifndef SCIMBRIDGEIMCONTEXT_H_
 #define SCIMBRIDGEIMCONTEXT_H_
 
+enum _scim_bridge_preedit_mode_t
+{
+    PREEDIT_FLOATING,
+    PREEDIT_HANGING,
+    PREEDIT_EMBEDDED,
+    PREEDIT_ANY,
+};
+
+/**
+ * The type for the way to show the preedit.
+ */
+typedef enum _scim_bridge_preedit_mode_t scim_bridge_preedit_mode_t;
+
 /**
  * This is the type for imcontext id.
  * Notice, every imcontext has the unique id.
