@@ -33,11 +33,6 @@
  */
 typedef struct _ScimBridgeMessage ScimBridgeMessage;
 
-/**
- * The id of responses. This will be duplicated in 0.4.0.
- */
-typedef unsigned char scim_bridge_message_response_id_t;
-
 #ifdef __cplusplus
 extern "C"
 {
