@@ -53,16 +53,13 @@ extern "C"
     /**
      * Print out an debug message and give a line feed ('\n').
      * The error messages will be shown in the stdout.
-     * Note, the message will be ignored when the given debug flag has not been stored,
-     * or the given debug level is lower than that of the preference.
-     * @ ee scim_bridge_debug_get_flag ()
+     * Note, the message will be ignored if the given debug level is lower than that of the preference.
      * @ ee scim_bridge_debug_get_level ()
      *
-     * @param flag The debug flag.
      * @param level The debug level.
      * @param format The message in printf style.
      */
-    void scim_bridge_pdebugln (scim_bridge_debug_flag_t flag, scim_bridge_debug_level_t level, const char *format,...);
+    void scim_bridge_pdebugln (scim_bridge_debug_level_t level, const char *format,...);
 
     /**
      * Print out a message without giving a line feed ('\n').
@@ -83,16 +80,13 @@ extern "C"
     /**
      * Print out an debug message without giving a line feed ('\n').
      * The error messages will be shown in the stdout.
-     * Note, the message will be ignored when the given debug flag has not been stored,
-     * or the given debug level is lower than that of the preference.
-     * @ ee scim_bridge_debug_get_flag ()
+     * Note, the message will be ignored if the given debug level is lower than that of the preference.
      * @ ee scim_bridge_debug_get_level ()
      *
-     * @param flag The debug flag.
      * @param level The debug level.
      * @param format The message in printf style.
      */
-    void scim_bridge_pdebug (scim_bridge_debug_flag_t flag, scim_bridge_debug_level_t level, const char *format,...);
+    void scim_bridge_pdebug (scim_bridge_debug_level_t level, const char *format,...);
 
 #ifdef __cplusplus
 }
