@@ -21,7 +21,7 @@
 /**
  * @file
  * @author Ryo Dairiki <ryo-dairiki@users.sourceforge.net>
- * @brief This header describes abut file pathes.
+ * @brief This header describes abut path related information.
  */
 #ifndef SCIMBRIDGEPATH_H_
 #define SCIMBRIDGEPATH_H_
@@ -41,12 +41,19 @@ extern "C"
     const char *scim_bridge_path_get_socket ();
 
     /**
-     * Get the lockfile path of the agent.
+     * Get the path for the lockfile of the agent.
      *
-     * @return The lockfile path.
+     * @return The path for the lockfile.
      */
     const char *scim_bridge_path_get_lockfile ();
 
+    /**
+     * Get the path for the binary of the agent.
+     *
+     * @return The path for the binary of the agent.
+     */
+    const char *scim_bridge_path_get_agent ();
+
 #ifdef __cplusplus
 }
 #endif
