@@ -26,17 +26,14 @@
 #include <string.h>
 #include <wchar.h>
 
-#include "scim-bridge-exception.h"
 #include "scim-bridge-string.h"
+#include "scim-bridge-output.h"
 
 ssize_t scim_bridge_string_to_wstring (wchar **wstr, const char *str)
 {
-    scim_bridge_exception_clear ();
-
     if (str == NULL) {
         *wstr = NULL;
-        scim_bridge_exception_occured (NULL_POINTER_EXCEPTION, "NULL pointer is given as the UTF8 string");
-        scim_bridge_exception_push_stack ("scim_bridge_string_to_wstring");
+        scim_bridge_perrorln ("A NULL pointer is given as the UTF8 string at scim_bridge_string_to_wstring ()");
         return RETVAL_FAILED;
     }
 
@@ -97,8 +94,7 @@ ssize_t scim_bridge_string_to_wstring (wchar **wstr, const char *str)
             case 2:
                 if (str_index + 1 > str_length || ! ((b ^ 0x80) < 0x40)) {
                     *wstr = NULL;
-                    scim_bridge_exception_occured (INVALID_UTF8_EXCEPTION, "Invalid UTF8 string is given");
-                    scim_bridge_exception_push_stack ("scim_bridge_string_to_wstring ()");
+                    scim_bridge_perrorln ("An invalid UTF8 string is given at scim_bridge_string_to_wstring ()");
                     return RETVAL_FAILED;
                 } else {
                     wstr_buffer[i] = ((wchar) (a & 0x1f) << 6)
@@ -111,8 +107,7 @@ ssize_t scim_bridge_string_to_wstring (wchar **wstr, const char *str)
                     || ! ((b ^ 0x80) < 0x40 && (c ^ 0x80) < 0x40
                     && (a >= 0xe1 || b >= 0xa0))) {
                     *wstr = NULL;
-                    scim_bridge_exception_occured (INVALID_UTF8_EXCEPTION, "Invalid UTF8 string is given");
-                    scim_bridge_exception_push_stack ("scim_bridge_string_to_wstring ()");
+                    scim_bridge_perrorln ("An invalid UTF8 string is given at scim_bridge_string_to_wstring ()");
                     return RETVAL_FAILED;
                 } else {
                     wstr_buffer[i] = ((wchar) (a & 0x0f) << 12)
@@ -126,8 +121,7 @@ ssize_t scim_bridge_string_to_wstring (wchar **wstr, const char *str)
                     || ! ((b ^ 0x80) < 0x40 && (c ^ 0x80) < 0x40
                     && (d ^ 0x80) < 0x40 && (a >= 0xf1 || b >= 0x90))) {
                     *wstr = NULL;
-                    scim_bridge_exception_occured (INVALID_UTF8_EXCEPTION, "Invalid UTF8 string is given");
-                    scim_bridge_exception_push_stack ("scim_bridge_string_to_wstring ()");
+                    scim_bridge_perrorln ("An invalid UTF8 string is given at scim_bridge_string_to_wstring ()");
                     return RETVAL_FAILED;
                 } else {
                     wstr_buffer[i] = ((wchar) (a & 0x07) << 18)
@@ -143,8 +137,7 @@ ssize_t scim_bridge_string_to_wstring (wchar **wstr, const char *str)
                     && (d ^ 0x80) < 0x40 && (e ^ 0x80) < 0x40
                     && (a >= 0xf9 || b >= 0x88))) {
                     *wstr = NULL;
-                    scim_bridge_exception_occured (INVALID_UTF8_EXCEPTION, "Invalid UTF8 string is given");
-                    scim_bridge_exception_push_stack ("scim_bridge_string_to_wstring ()");
+                    scim_bridge_perrorln ("An invalid UTF8 string is given at scim_bridge_string_to_wstring ()");
                     return RETVAL_FAILED;
                 } else {
                     wstr_buffer[i] = ((wchar) (a & 0x03) << 24)
@@ -161,8 +154,7 @@ ssize_t scim_bridge_string_to_wstring (wchar **wstr, const char *str)
                     && (d ^ 0x80) < 0x40 && (e ^ 0x80) < 0x40
                     && (f ^ 0x80) < 0x40 && (a >= 0xfd || b >= 0x84))) {
                     *wstr = NULL;
-                    scim_bridge_exception_occured (INVALID_UTF8_EXCEPTION, "Invalid UTF8 string is given");
-                    scim_bridge_exception_push_stack ("scim_bridge_string_to_wstring ()");
+                    scim_bridge_perrorln ("An invalid UTF8 string is given at scim_bridge_string_to_wstring ()");
                     return RETVAL_FAILED;
                 } else {
                     wstr_buffer[i] = ((wchar) (a & 0x01) << 30)
@@ -175,8 +167,7 @@ ssize_t scim_bridge_string_to_wstring (wchar **wstr, const char *str)
                 }
             default:
                 *wstr = NULL;
-                scim_bridge_exception_occured (INVALID_UTF8_EXCEPTION, "Invalid UTF8 string is given");
-                scim_bridge_exception_push_stack ("scim_bridge_string_to_wstring ()");
+                scim_bridge_perrorln ("An invalid UTF8 string is given at scim_bridge_string_to_wstring ()");
                 return RETVAL_FAILED;
         }
     }
@@ -190,12 +181,9 @@ ssize_t scim_bridge_string_to_wstring (wchar **wstr, const char *str)
 
 ssize_t scim_bridge_wstring_to_string (char **str, const wchar *wstr)
 {
-    scim_bridge_exception_clear ();
-
     if (wstr == NULL) {
         *str = NULL;
-        scim_bridge_exception_occured (NULL_POINTER_EXCEPTION, "NULL pointer is given as the UCS4 string");
-        scim_bridge_exception_push_stack ("scim_bridge_wstring_to_string ()");
+        scim_bridge_perrorln ("A NULL pointer is given as the UCS4 string at scim_bridge_wstring_to_string ()");
         return RETVAL_FAILED;
     }
 
@@ -228,8 +216,7 @@ ssize_t scim_bridge_wstring_to_string (char **str, const wchar *wstr)
 
         if (wc_size_in_utf8 == 0) {
             *str = NULL;
-            scim_bridge_exception_occured (INVALID_UCS4_EXCEPTION, "Invalid wchar is given: %lc", wc);
-            scim_bridge_exception_push_stack ("scim_bridge_wstring_to_string ()");
+            scim_bridge_perrorln ("An invalid UCS4 string is given at scim_bridge_wstring_to_string ()");
             return RETVAL_FAILED;
         }
 
@@ -261,11 +248,8 @@ ssize_t scim_bridge_wstring_to_string (char **str, const wchar *wstr)
 
 ssize_t scim_bridge_wstring_get_length (const wchar *wstr)
 {
-    scim_bridge_exception_clear ();
-
     if (wstr == NULL) {
-        scim_bridge_exception_occured (NULL_POINTER_EXCEPTION, "NULL pointer is given as the UCS4 string");
-        scim_bridge_exception_push_stack ("scim_bridge_wstring_get_length ()");
+        scim_bridge_perrorln ("A NULL pointer is given as the UCS4 string at scim_bridge_wstring_get_length ()");
         return RETVAL_FAILED;
     }
 
@@ -278,11 +262,8 @@ ssize_t scim_bridge_wstring_get_length (const wchar *wstr)
 
 ssize_t scim_bridge_string_get_length (const char *str)
 {
-    scim_bridge_exception_clear ();
-
     if (str == NULL) {
-        scim_bridge_exception_occured (NULL_POINTER_EXCEPTION, "NULL pointer is given as the UTF8 string");
-        scim_bridge_exception_push_stack ("scim_bridge_string_get_length ()");
+        scim_bridge_perrorln ("A NULL pointer is given as the UTF8 string at scim_bridge_string_get_length ()");
         return RETVAL_FAILED;
     }
 
@@ -292,11 +273,8 @@ ssize_t scim_bridge_string_get_length (const char *str)
 
 retval_t scim_bridge_string_to_uint (unsigned int *dst, const char *str)
 {
-    scim_bridge_exception_clear ();
-
     if (str == NULL) {
-        scim_bridge_exception_occured (NULL_POINTER_EXCEPTION, "NULL pointer is given as a string");
-        scim_bridge_exception_push_stack ("scim_bridge_string_to_uint ()");
+        scim_bridge_perrorln ("A NULL pointer is given as a string at scim_bridge_string_to_uint ()");
         return RETVAL_FAILED;
     } else {
         unsigned long value = 0;
@@ -336,15 +314,13 @@ retval_t scim_bridge_string_to_uint (unsigned int *dst, const char *str)
                     digit = 9;
                     break;
                 default:
-                    scim_bridge_exception_occured (NUMBER_FORMAT_EXCEPTION, "An invalid char is given: %c", str[i]);
-                    scim_bridge_exception_push_stack ("scim_bridge_string_to_uint ()");
+                    scim_bridge_perrorln ("An invalid char is given at scim_bridge_string_to_uint (): %c", str[i]);
                     return RETVAL_FAILED;
             }
             value *= 10;
             value += digit;
             if (value > UINT_MAX) {
-                scim_bridge_exception_occured (OVER_FLOW_EXCEPTION, "Over flow exception");
-                scim_bridge_exception_push_stack ("scim_bridge_string_to_uint ()");
+                scim_bridge_perrorln ("An over flow exception occurred at scim_bridge_string_to_uint ()");
                 return RETVAL_FAILED;
             }
         }
@@ -357,11 +333,8 @@ retval_t scim_bridge_string_to_uint (unsigned int *dst, const char *str)
 
 retval_t scim_bridge_string_to_int (int *dst, const char *str)
 {
-    scim_bridge_exception_clear ();
-
     if (str == NULL) {
-        scim_bridge_exception_occured (NULL_POINTER_EXCEPTION, "NULL pointer is given as a string");
-        scim_bridge_exception_push_stack ("scim_bridge_string_to_int ()");
+        scim_bridge_perrorln ("A NULL pointer is given as a string at scim_bridge_string_to_int ()");
         return RETVAL_FAILED;
     } else {
         unsigned long value = 0;
@@ -375,8 +348,7 @@ retval_t scim_bridge_string_to_int (int *dst, const char *str)
                     if (i == 0) {
                         negative = TRUE;
                     } else {
-                        scim_bridge_exception_occured (NUMBER_FORMAT_EXCEPTION, "negatives sign cannot be given at the middle of the number: %s", str);
-                        scim_bridge_exception_push_stack ("scim_bridge_string_to_int ()");
+                        scim_bridge_perrorln ("Negative signs cannot be given at the middle of the number at scim_bridge_string_to_int (): %s", str);
                         return RETVAL_FAILED;
                     }
                 case '0':
@@ -410,22 +382,19 @@ retval_t scim_bridge_string_to_int (int *dst, const char *str)
                     digit = 9;
                     break;
                 default:
-                    scim_bridge_exception_occured (NUMBER_FORMAT_EXCEPTION, "An invalid char is given: %c", str[i]);
-                    scim_bridge_exception_push_stack ("scim_bridge_string_to_int ()");
+                    scim_bridge_perrorln ("An invalid char is given at scim_bridge_string_to_int (): %c", str[i]);
                     return RETVAL_FAILED;
             }
             value *= 10;
             value += digit;
             if (!negative) {
                 if (value > INT_MAX) {
-                    scim_bridge_exception_occured (OVER_FLOW_EXCEPTION, "Over flow exception");
-                    scim_bridge_exception_push_stack ("scim_bridge_string_to_uint ()");
+                    scim_bridge_perrorln ("An over flow exception occurred at scim_bridge_string_to_int ()");
                     return RETVAL_FAILED;
                 }
             } else {
-                if (-value < INT_MIN) {
-                    scim_bridge_exception_occured (OVER_FLOW_EXCEPTION, "Over flow exception");
-                    scim_bridge_exception_push_stack ("scim_bridge_string_to_int ()");
+                if (-((long) value) < INT_MIN) {
+                    scim_bridge_perrorln ("An over flow exception at scim_bridge_string_to_int ()");
                     return RETVAL_FAILED;
                 }
             }
@@ -443,11 +412,8 @@ retval_t scim_bridge_string_to_int (int *dst, const char *str)
 
 retval_t scim_bridge_string_to_boolean (boolean *dst, const char *str)
 {
-    scim_bridge_exception_clear ();
-
     if (str == NULL) {
-        scim_bridge_exception_occured (NULL_POINTER_EXCEPTION, "NULL pointer is given as a string");
-        scim_bridge_exception_push_stack ("scim_bridge_string_to_boolean ()");
+        scim_bridge_perrorln ("A NULL pointer is given as a string at scim_bridge_string_to_boolean ()");
         return RETVAL_FAILED;
     } else {
         if (strcmp (str, "TRUE") == 0 || strcmp (str, "True") == 0 || strcmp (str, "true") == 0) {
@@ -457,8 +423,7 @@ retval_t scim_bridge_string_to_boolean (boolean *dst, const char *str)
             *dst = FALSE;
             return RETVAL_SUCCEEDED;
         } else {
-            scim_bridge_exception_occured (NUMBER_FORMAT_EXCEPTION, "An invalid string is given: %s", str);
-            scim_bridge_exception_push_stack ("scim_bridge_string_to_boolean ()");
+            scim_bridge_perrorln ("An invalid string is given at scim_bridge_string_to_boolean (): %s", str);
             return RETVAL_FAILED;
         }
     }
@@ -487,11 +452,11 @@ size_t scim_bridge_string_from_boolean (char **str, boolean value)
 {
     if (value == TRUE) {
         *str = malloc (sizeof (char) * 5);
-        strcpy (*str, "true");
+        strcpy (*str, "TRUE");
         return 4;
     } else {
         *str = malloc (sizeof (char) * 6);
-        strcpy (*str, "false");
+        strcpy (*str, "FALSE");
         return 5;
     }
 }
