@@ -18,15 +18,21 @@
  * Boston, MA  02111-1307  USA
  */
 
-#ifndef SCIMBRIDCOMMON_H_
-#define SCIMBRIDCOMMON_H_
+/**
+ * @file
+ * @author Ryo Dairiki <ryo-dairiki@users.sourceforge.net>
+ * @brief This header describes about fundamental definitions of scim-bridge.
+ */
 
-#include <signal.h>
+#ifndef SCIMBRIDE_H_
+#define SCIMBRIDE_H_
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <sys/types.h>
+
 #ifndef SCIM_VERSION
 #define SCIM_VERSION "<unknown>"
 #endif
@@ -35,8 +41,6 @@
 #define N_(String)  gettext_noop(String)
 #define gettext_noop(String) (String)
 
-#define SIGNOTIFY SIGUSR1
-
 #ifndef NPOS
 #define NPOS -1
 #endif
@@ -69,7 +73,12 @@ typedef int retval_t;
 #define RETVAL_FAILED -1
 
 /**
- * The return value of success.
+ * The return value of successness.
  */
 #define RETVAL_SUCCEEDED 0
-#endif                                            /*SCIMBRIDCOMMON_H_*/
+
+/**
+ * The return value of ignoreness.
+ */
+#define RETVAL_IGNORED 1
+#endif                                            /*SCIMBRIDE_H_*/
